@@ -54,52 +54,37 @@ extern struct TALER_Amount TALER_ARL_currency_round_unit;
  */
 extern const struct GNUNET_CONFIGURATION_Handle *TALER_ARL_cfg;
 
-/**
- * Our session with the #TALER_ARL_edb.
- */
-extern struct TALER_EXCHANGEDB_Session *TALER_ARL_esession;
-
 /**
  * Handle to access the auditor's database.
  */
 extern struct TALER_AUDITORDB_Plugin *TALER_ARL_adb;
 
-/**
- * Our session with the #TALER_ARL_adb.
- */
-extern struct TALER_AUDITORDB_Session *TALER_ARL_asession;
-
 /**
  * Master public key of the exchange to audit.
  */
 extern struct TALER_MasterPublicKeyP TALER_ARL_master_pub;
 
+/**
+ * Public key of the auditor.
+ */
+extern struct TALER_AuditorPublicKeyP TALER_ARL_auditor_pub;
+
+/**
+ * REST API endpoint of the auditor.
+ */
+extern char *TALER_ARL_auditor_url;
+
+/**
+ * REST API endpoint of the exchange.
+ */
+extern char *TALER_ARL_exchange_url;
+
 /**
  * At what time did the auditor process start?
  */
 extern struct GNUNET_TIME_Absolute start_time;
 
 
-/**
- * Convert absolute time to human-readable JSON string.
- *
- * @param at time to convert
- * @return human-readable string representing the time
- */
-json_t *
-TALER_ARL_json_from_time_abs_nbo (struct GNUNET_TIME_AbsoluteNBO at);
-
-
-/**
- * Convert absolute time to human-readable JSON string.
- *
- * @param at time to convert
- * @return human-readable string representing the time
- */
-json_t *
-TALER_ARL_json_from_time_abs (struct GNUNET_TIME_Absolute at);
-
-
 /**
  * Add @a object to the report @a array.  Fail hard if this fails.
  *
@@ -121,8 +106,8 @@ TALER_ARL_report (json_t *array,
  */
 enum GNUNET_DB_QueryStatus
 TALER_ARL_get_denomination_info_by_hash (
-  const struct GNUNET_HashCode *dh,
-  const struct TALER_DenominationKeyValidityPS **issue);
+  const struct TALER_DenominationHashP *dh,
+  const struct TALER_EXCHANGEDB_DenominationKeyInformation **issue);
 
 
 /**
@@ -137,8 +122,8 @@ TALER_ARL_get_denomination_info_by_hash (
 enum GNUNET_DB_QueryStatus
 TALER_ARL_get_denomination_info (
   const struct TALER_DenominationPublicKey *denom_pub,
-  const struct TALER_DenominationKeyValidityPS **issue,
-  struct GNUNET_HashCode *dh);
+  const struct TALER_EXCHANGEDB_DenominationKeyInformation **issue,
+  struct TALER_DenominationHashP *dh);
 
 
 /**
@@ -296,7 +281,7 @@ TALER_ARL_amount_subtract_neg_ (struct TALER_Amount *diff,
  * @param ana_cls closure for @a ana
  * @return #GNUNET_OK on success
  */
-int
+enum GNUNET_GenericReturnValue
 TALER_ARL_setup_sessions_and_run (TALER_ARL_Analysis ana,
                                   void *ana_cls);
 
@@ -318,7 +303,7 @@ TALER_ARL_do_abort (void);
  * @param c configuration to use
  * @return #GNUNET_OK on success
  */
-int
+enum GNUNET_GenericReturnValue
 TALER_ARL_init (const struct GNUNET_CONFIGURATION_Handle *c);
 
 
