@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2015, 2016, 2017 GNUnet e.V.
+  Copyright (C) 2015, 2016, 2017 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -21,7 +21,7 @@
  */
 #ifndef BANK_API_COMMON_H
 #define BANK_API_COMMON_H
-#include <jansson.h>
+
 #include <gnunet/gnunet_util_lib.h>
 #include <gnunet/gnunet_json_lib.h>
 #include <gnunet/gnunet_curl_lib.h>
@@ -30,35 +30,15 @@
 
 
 /**
- * Build authentication header from @a auth.
+ * Set authentication data in @a easy from @a auth.
  *
+ * @param easy curl handle to setup for authentication
  * @param auth authentication data to use
- * @return NULL on error, otherwise curl headers to use
+ * @return #GNUNET_OK in success
  */
-struct curl_slist *
-TALER_BANK_make_auth_header_ (const struct TALER_BANK_AuthenticationData *auth);
-
-
-/**
- * Obtain the URL to use for an API request.
- *
- * @param u base URL of the bank
- * @param path Taler API path (i.e. "/history")
- * @return the full URL to use with cURL
- */
-char *
-TALER_BANK_path_to_url_ (const char *u,
-                         const char *path);
-
-
-/**
- * Parse error code given in @a json.
- *
- * @param json the json to parse
- * @return error code, or #TALER_EC_INVALID if not found
- */
-enum TALER_ErrorCode
-TALER_BANK_parse_ec_ (const json_t *json);
+int
+TALER_BANK_setup_auth_ (CURL *easy,
+                        const struct TALER_BANK_AuthenticationData *auth);
 
 
 #endif
