@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2019-2020 Taler Systems SA
+  Copyright (C) 2019-2021 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published
@@ -24,20 +24,13 @@
 #include "platform.h"
 #include "taler_curl_lib.h"
 
-#if COMPRESS_BODIES
+
+#if TALER_CURL_COMPRESS_BODIES
 #include <zlib.h>
 #endif
 
 
-/**
- * Add the @a body as POST data to the easy handle in @a ctx.
- *
- * @param[in,out] ctx a request context (updated)
- * @param eh easy handle to use
- * @param body JSON body to add to @e ctx
- * @return #GNUNET_OK on success #GNUNET_SYSERR on failure
- */
-int
+enum GNUNET_GenericReturnValue
 TALER_curl_easy_post (struct TALER_CURL_PostContext *ctx,
                       CURL *eh,
                       const json_t *body)
@@ -53,7 +46,8 @@ TALER_curl_easy_post (struct TALER_CURL_PostContext *ctx,
     return GNUNET_SYSERR;
   }
   slen = strlen (str);
-#if COMPRESS_BODIES
+  if (TALER_CURL_COMPRESS_BODIES &&
+      (! ctx->disable_compression) )
   {
     Bytef *cbuf;
     uLongf cbuf_size;
@@ -75,19 +69,21 @@ TALER_curl_easy_post (struct TALER_CURL_PostContext *ctx,
     free (str);
     slen = (size_t) cbuf_size;
     ctx->json_enc = (char *) cbuf;
+    GNUNET_assert (
+      NULL !=
+      (ctx->headers = curl_slist_append (
+         ctx->headers,
+         "Content-Encoding: deflate")));
   }
-  GNUNET_assert
-    (NULL != (ctx->headers = curl_slist_append
-                               (ctx->headers,
-                               "Content-Encoding: deflate")));
-#else
-  ctx->json_enc = str;
-#endif
-
-  GNUNET_assert
-    (NULL != (ctx->headers = curl_slist_append
-                               (ctx->headers,
-                               "Content-Type: application/json")));
+  else
+  {
+    ctx->json_enc = str;
+  }
+  GNUNET_assert (
+    NULL !=
+    (ctx->headers = curl_slist_append (
+       ctx->headers,
+       "Content-Type: application/json")));
 
   GNUNET_assert (CURLE_OK ==
                  curl_easy_setopt (eh,
@@ -101,11 +97,6 @@ TALER_curl_easy_post (struct TALER_CURL_PostContext *ctx,
 }
 
 
-/**
- * Free the data in @a ctx.
- *
- * @param[in] ctx a request context (updated)
- */
 void
 TALER_curl_easy_post_finished (struct TALER_CURL_PostContext *ctx)
 {
