@@ -82,6 +82,11 @@ extern bool TEH_suicide;
  */
 extern struct TALER_MasterPublicKeyP TEH_master_public_key;
 
+/**
+ * Key used to encrypt KYC attribute data in our database.
+ */
+extern struct TALER_AttributeEncryptionKeyP TEH_attribute_key;
+
 /**
  * Our DB plugin.
  */
@@ -92,6 +97,13 @@ extern struct TALER_EXCHANGEDB_Plugin *TEH_plugin;
  */
 extern char *TEH_currency;
 
+/**
+ * What is the largest amount we allow a peer to
+ * merge into a reserve before always triggering
+ * an AML check?
+ */
+extern struct TALER_Amount TEH_aml_threshold;
+
 /**
  * Our (externally visible) base URL.
  */
@@ -213,7 +225,7 @@ struct TEH_RequestHandler
      *
      * @param rc context for the request
      * @param json uploaded JSON data
-     * @param args array of arguments, needs to be of length @e args_expected
+     * @param args array of arguments, needs to be of length @e nargs
      * @return MHD result code
      */
     MHD_RESULT
@@ -225,7 +237,7 @@ struct TEH_RequestHandler
      * Function to call to handle DELETE requests.
      *
      * @param rc context for the request
-     * @param args array of arguments, needs to be of length @e args_expected
+     * @param args array of arguments, needs to be of length @e nargs
      * @return MHD result code
      */
     MHD_RESULT
@@ -234,17 +246,6 @@ struct TEH_RequestHandler
 
   } handler;
 
-  /**
-   * Number of arguments this handler expects in the @a args array.
-   */
-  unsigned int nargs;
-
-  /**
-   * Is the number of arguments given in @e nargs only an upper bound,
-   * and calling with fewer arguments could be OK?
-   */
-  bool nargs_is_upper_bound;
-
   /**
    * Mime type to use in reply (hint, can be NULL).
    */
@@ -264,6 +265,17 @@ struct TEH_RequestHandler
    * Default response code. 0 for none provided.
    */
   unsigned int response_code;
+
+  /**
+   * Number of arguments this handler expects in the @a args array.
+   */
+  unsigned int nargs;
+
+  /**
+   * Is the number of arguments given in @e nargs only an upper bound,
+   * and calling with fewer arguments could be OK?
+   */
+  bool nargs_is_upper_bound;
 };
 
 
