@@ -24,6 +24,7 @@
 #include <jansson.h>
 #include <microhttpd.h>
 #include <pthread.h>
+#include "taler_attributes.h"
 #include "taler_json_lib.h"
 #include "taler_kyclogic_lib.h"
 #include "taler_mhd_lib.h"
@@ -169,6 +170,7 @@ TEH_kyc_proof_cleanup (void)
  * @param provider_user_id set to user ID at the provider, or NULL if not supported or unknown
  * @param provider_legitimization_id set to legitimization process ID at the provider, or NULL if not supported or unknown
  * @param expiration until when is the KYC check valid
+ * @param attributes user attributes returned by the provider
  * @param http_status HTTP status code of @a response
  * @param[in] response to return to the HTTP client
  */
@@ -179,6 +181,7 @@ proof_cb (
   const char *provider_user_id,
   const char *provider_legitimization_id,
   struct GNUNET_TIME_Absolute expiration,
+  const json_t *attributes,
   unsigned int http_status,
   struct MHD_Response *response)
 {
@@ -193,7 +196,41 @@ proof_cb (
   if (TALER_KYCLOGIC_STATUS_SUCCESS == status)
   {
     enum GNUNET_DB_QueryStatus qs;
+    size_t eas;
+    void *ea;
+    const char *birthdate;
+    struct GNUNET_ShortHashCode kyc_prox;
 
+    TALER_CRYPTO_attributes_to_kyc_prox (attributes,
+                                         &kyc_prox);
+    birthdate = json_string_value (json_object_get (attributes,
+                                                    TALER_ATTRIBUTE_BIRTHDATE));
+    TALER_CRYPTO_kyc_attributes_encrypt (&TEH_attribute_key,
+                                         attributes,
+                                         &ea,
+                                         &eas);
+    qs = TEH_plugin->insert_kyc_attributes (
+      TEH_plugin->cls,
+      &kpc->h_payto,
+      &kyc_prox,
+      kpc->provider_section,
+      birthdate,
+      GNUNET_TIME_timestamp_get (),
+      GNUNET_TIME_absolute_to_timestamp (expiration),
+      eas,
+      ea);
+    GNUNET_free (ea);
+    if (GNUNET_DB_STATUS_HARD_ERROR == qs)
+    {
+      GNUNET_break (0);
+      if (NULL != response)
+        MHD_destroy_response (response);
+      kpc->response_code = MHD_HTTP_INTERNAL_SERVER_ERROR;
+      kpc->response = TALER_MHD_make_error (TALER_EC_GENERIC_DB_STORE_FAILED,
+                                            "insert_kyc_attributes");
+      GNUNET_async_scope_restore (&old_scope);
+      return;
+    }
     qs = TEH_plugin->update_kyc_process_by_row (TEH_plugin->cls,
                                                 kpc->process_row,
                                                 kpc->provider_section,
