@@ -215,10 +215,14 @@ check_commitment (struct RevealContext *rctx,
     case TALER_DENOMINATION_CS:
       {
         enum TALER_ErrorCode ec;
+        const struct TEH_CsDeriveData cdd = {
+          .h_denom_pub = &rctx->rrcs[j].h_denom_pub,
+          .nonce = &nonces[aoff]
+        };
 
-        ec = TEH_keys_denomination_cs_r_pub_melt (
-          &rctx->rrcs[j].h_denom_pub,
-          &nonces[aoff],
+        ec = TEH_keys_denomination_cs_r_pub (
+          &cdd,
+          true,
           &alg_values->details.cs_values);
         if (TALER_EC_NONE != ec)
         {
@@ -746,16 +750,21 @@ clean_age:
               (unsigned int) rctx->num_fresh_coins);
 
   /* create fresh coin signatures */
-  for (unsigned int i = 0; i<rctx->num_fresh_coins; i++)
   {
+    struct TEH_CoinSignData csds[rctx->num_fresh_coins];
+    struct TALER_BlindedDenominationSignature bss[rctx->num_fresh_coins];
     enum TALER_ErrorCode ec;
 
-    // FIXME #7272: replace with a batch call that
-    // passes all coins in once go!
-    ec = TEH_keys_denomination_sign_melt (
-      &rrcs[i].h_denom_pub,
-      &rcds[i].blinded_planchet,
-      &rrcs[i].coin_sig);
+    for (unsigned int i = 0; i<rctx->num_fresh_coins; i++)
+    {
+      csds[i].h_denom_pub = &rrcs[i].h_denom_pub;
+      csds[i].bp = &rcds[i].blinded_planchet;
+    }
+    ec = TEH_keys_denomination_batch_sign (
+      csds,
+      rctx->num_fresh_coins,
+      true,
+      bss);
     if (TALER_EC_NONE != ec)
     {
       GNUNET_break (0);
@@ -764,8 +773,9 @@ clean_age:
                                      NULL);
       goto cleanup;
     }
+    for (unsigned int i = 0; i<rctx->num_fresh_coins; i++)
+      rrcs[i].coin_sig = bss[i];
   }
-
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Signatures ready, starting DB interaction\n");
 
