@@ -189,24 +189,47 @@ purse_transaction (void *cls,
 {
   struct ReservePurseContext *rpc = cls;
   enum GNUNET_DB_QueryStatus qs;
+  char *required;
 
-  const char *required;
-
-  required = TALER_KYCLOGIC_kyc_test_required (
+  qs = TALER_KYCLOGIC_kyc_test_required (
     TALER_KYCLOGIC_KYC_TRIGGER_P2P_RECEIVE,
     &rpc->h_payto,
     TEH_plugin->select_satisfied_kyc_processes,
     TEH_plugin->cls,
     &amount_iterator,
-    rpc);
+    rpc,
+    &required);
+  if (qs < 0)
+  {
+    if (GNUNET_DB_STATUS_SOFT_ERROR == qs)
+      return qs;
+    GNUNET_break (0);
+    *mhd_ret =
+      TALER_MHD_reply_with_error (connection,
+                                  MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                  TALER_EC_GENERIC_DB_FETCH_FAILED,
+                                  "kyc_test_required");
+    return GNUNET_DB_STATUS_HARD_ERROR;
+  }
   if (NULL != required)
   {
     rpc->kyc.ok = false;
-    return TEH_plugin->insert_kyc_requirement_for_account (
+    qs = TEH_plugin->insert_kyc_requirement_for_account (
       TEH_plugin->cls,
       required,
       &rpc->h_payto,
       &rpc->kyc.requirement_row);
+    GNUNET_free (required);
+    if (GNUNET_DB_STATUS_HARD_ERROR == qs)
+    {
+      GNUNET_break (0);
+      *mhd_ret
+        = TALER_MHD_reply_with_error (connection,
+                                      MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                      TALER_EC_GENERIC_DB_STORE_FAILED,
+                                      "insert_kyc_requirement_for_account");
+    }
+    return qs;
   }
   rpc->kyc.ok = true;
 
@@ -230,8 +253,7 @@ purse_transaction (void *cls,
     {
       if (GNUNET_DB_STATUS_SOFT_ERROR == qs)
         return qs;
-      TALER_LOG_WARNING (
-        "Failed to store purse purse information in database\n");
+      GNUNET_break (0);
       *mhd_ret =
         TALER_MHD_reply_with_error (connection,
                                     MHD_HTTP_INTERNAL_SERVER_ERROR,
