@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2020 Taler Systems SA
+  Copyright (C) 2014-2021 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -28,24 +28,20 @@
 
 
 /**
- * Handle a "/reserves/$RESERVE_PUB/withdraw" request.  Parses the
- * "reserve_pub" EdDSA key of the reserve and the requested "denom_pub" which
+ * Handle a "/reserves/$RESERVE_PUB/withdraw" request.  Parses the requested "denom_pub" which
  * specifies the key/value of the coin to be withdrawn, and checks that the
  * signature "reserve_sig" makes this a valid withdrawal request from the
  * specified reserve.  If so, the envelope with the blinded coin "coin_ev" is
  * passed down to execute the withdrawal operation.
  *
- * @param rh context of the handler
- * @param connection the MHD connection to handle
+ * @param rc request context
  * @param root uploaded JSON data
- * @param args array of additional options (first must be the
- *         reserve public key, the second one should be "withdraw")
+ * @param reserve_pub public key of the reserve
  * @return MHD result code
   */
 MHD_RESULT
-TEH_handler_withdraw (const struct TEH_RequestHandler *rh,
-                      struct MHD_Connection *connection,
-                      const json_t *root,
-                      const char *const args[2]);
+TEH_handler_withdraw (struct TEH_RequestContext *rc,
+                      const struct TALER_ReservePublicKeyP *reserve_pub,
+                      const json_t *root);
 
 #endif
