@@ -51,6 +51,7 @@
 #include "pg_lookup_records_by_table.h"
 #include "pg_lookup_serial_by_table.h"
 #include "pg_select_account_merges_above_serial_id.h"
+#include "pg_select_aml_threshold.h"
 #include "pg_select_all_purse_decisions_above_serial_id.h"
 #include "pg_select_purse.h"
 #include "pg_select_purse_deposits_above_serial_id.h"
@@ -203,14 +204,12 @@
 #include "pg_reserves_update.h"
 #include "pg_setup_wire_target.h"
 #include "pg_compute_shard.h"
-#include "pg_batch_reserves_in_insert.h"
-#include "pg_batch2_reserves_in_insert.h"
 #include "pg_insert_kyc_attributes.h"
 #include "pg_update_kyc_attributes.h"
 #include "pg_select_similar_kyc_attributes.h"
 #include "pg_select_kyc_attributes.h"
 #include "pg_insert_aml_officer.h"
-#include "pg_update_aml_officer.h"
+#include "pg_test_aml_officer.h"
 #include "pg_lookup_aml_officer.h"
 #include "pg_trigger_aml_process.h"
 #include "pg_select_aml_process.h"
@@ -453,6 +452,8 @@ libtaler_plugin_exchangedb_postgres_init (void *cls)
     = &TEH_PG_select_account_merges_above_serial_id;
   plugin->select_all_purse_decisions_above_serial_id
     = &TEH_PG_select_all_purse_decisions_above_serial_id;
+  plugin->select_aml_threshold
+    = &TEH_PG_select_aml_threshold;
   plugin->select_purse
     = &TEH_PG_select_purse;
   plugin->select_purse_deposits_above_serial_id
@@ -747,10 +748,6 @@ libtaler_plugin_exchangedb_postgres_init (void *cls)
     = &TEH_PG_select_purse_by_merge_pub;
   plugin->set_purse_balance
     = &TEH_PG_set_purse_balance;
-  plugin->batch_reserves_in_insert
-    = &TEH_PG_batch_reserves_in_insert;
-  plugin->batch2_reserves_in_insert
-    = &TEH_PG_batch2_reserves_in_insert;
   plugin->insert_kyc_attributes
     = &TEH_PG_insert_kyc_attributes;
   plugin->update_kyc_attributes
@@ -761,8 +758,8 @@ libtaler_plugin_exchangedb_postgres_init (void *cls)
     = &TEH_PG_select_kyc_attributes;
   plugin->insert_aml_officer
     = &TEH_PG_insert_aml_officer;
-  plugin->update_aml_officer
-    = &TEH_PG_update_aml_officer;
+  plugin->test_aml_officer
+    = &TEH_PG_test_aml_officer;
   plugin->lookup_aml_officer
     = &TEH_PG_lookup_aml_officer;
   plugin->trigger_aml_process
