@@ -1200,13 +1200,18 @@ THEN
   FROM exchange.purse_deposits
   WHERE coin_pub = in_coin_pub
     AND purse_pub = in_purse_pub
-    AND coin_sig = in_cion_sig;
+    AND coin_sig = in_coin_sig;
   IF NOT FOUND
   THEN
     out_balance_ok=FALSE;
     out_late=FALSE;
     out_conflict=TRUE;
     RETURN;
+  ELSE
+    out_late=FALSE;
+    out_balance_ok=TRUE;
+    out_conflict=FALSE;
+    RETURN;
   END IF;
 END IF;
 PERFORM
@@ -2104,6 +2109,154 @@ BEGIN
        WHERE
                policy_details_serial_id = out_policy_details_serial_id;
 END $$;
+CREATE OR REPLACE FUNCTION exchange_do_insert_aml_decision(
+  IN in_h_payto BYTEA,
+  IN in_new_threshold_val INT8,
+  IN in_new_threshold_frac INT4,
+  IN in_new_status INT4,
+  IN in_decision_time INT8,
+  IN in_justification VARCHAR,
+  IN in_decider_pub BYTEA,
+  IN in_decider_sig BYTEA,
+  IN in_notify_s VARCHAR,
+  IN in_kyc_requirements VARCHAR,
+  IN in_requirement_row INT8,
+  OUT out_invalid_officer BOOLEAN,
+  OUT out_last_date INT8)
+LANGUAGE plpgsql
+AS $$
+BEGIN
+PERFORM
+  FROM exchange.aml_staff
+  WHERE decider_pub=in_decider_pub
+    AND is_active
+    AND NOT read_only;
+IF NOT FOUND
+THEN
+  out_invalid_officer=TRUE;
+  out_last_date=0;
+  RETURN;
+END IF;
+out_invalid_officer=FALSE;
+SELECT decision_time
+  INTO out_last_date
+  FROM exchange.aml_history
+  WHERE h_payto=in_h_payto
+  ORDER BY decision_time DESC;
+IF FOUND
+THEN
+  IF out_last_date >= in_decision_time
+  THEN
+    RETURN;
+  END IF;
+  UPDATE exchange.aml_status
+    SET threshold_val=in_new_threshold_val
+       ,threshold_frac=in_new_threshold_frac
+       ,status=in_new_status
+       ,kyc_requirement=in_requirement_row
+   WHERE h_payto=in_h_payto;
+  ASSERT FOUND, 'cannot have AML decision history but no AML status';
+ELSE
+  out_last_date = 0;
+  INSERT INTO exchange.aml_status
+    (h_payto
+    ,threshold_val
+    ,threshold_frac
+    ,status
+    ,kyc_requirement)
+    VALUES
+    (in_h_payto
+    ,in_new_threshold_val
+    ,in_new_threshold_frac
+    ,in_new_status
+    ,in_requirement_row);
+END IF;
+INSERT INTO exchange.aml_history
+  (h_payto
+  ,new_threshold_val
+  ,new_threshold_frac
+  ,new_status
+  ,decision_time
+  ,justification
+  ,kyc_requirements
+  ,kyc_req_row
+  ,decider_pub
+  ,decider_sig
+  ) VALUES
+  (in_h_payto
+  ,in_new_threshold_val
+  ,in_new_threshold_frac
+  ,in_new_status
+  ,in_decision_time
+  ,in_justification
+  ,in_kyc_requirements
+  ,in_requirement_row
+  ,in_decider_pub
+  ,in_decider_sig);
+IF 0 = in_new_status
+THEN
+  INSERT INTO kyc_alerts
+    (h_payto
+    ,trigger_type)
+    VALUES
+    (in_h_payto,1);
+   EXECUTE FORMAT (
+     'NOTIFY %s'
+    ,in_notify_s);
+END IF;
+END $$;
+COMMENT ON FUNCTION exchange_do_insert_aml_decision(BYTEA, INT8, INT4, INT4, INT8, VARCHAR, BYTEA, BYTEA, VARCHAR, VARCHAR, INT8)
+  IS 'Checks whether the AML officer is eligible to make AML decisions and if so inserts the decision into the table';
+CREATE OR REPLACE FUNCTION exchange_do_insert_aml_officer(
+  IN in_decider_pub BYTEA,
+  IN in_master_sig BYTEA,
+  IN in_decider_name VARCHAR,
+  IN in_is_active BOOLEAN,
+  IN in_read_only BOOLEAN,
+  IN in_last_change INT8,
+  OUT out_last_change INT8)
+LANGUAGE plpgsql
+AS $$
+BEGIN
+INSERT INTO exchange.aml_staff
+  (decider_pub
+  ,master_sig
+  ,decider_name
+  ,is_active
+  ,read_only
+  ,last_change
+  ) VALUES
+  (in_decider_pub
+  ,in_master_sig
+  ,in_decider_name
+  ,in_is_active
+  ,in_read_only
+  ,in_last_change)
+ ON CONFLICT DO NOTHING;
+IF FOUND
+THEN
+  out_last_change=0;
+  RETURN;
+END IF;
+SELECT last_change
+  INTO out_last_change
+  FROM exchange.aml_staff
+  WHERE decider_pub=in_decider_pub;
+ASSERT FOUND, 'cannot have INSERT conflict but no AML staff record';
+IF out_last_change >= in_last_change
+THEN
+ RETURN;
+END IF;
+UPDATE exchange.aml_staff
+  SET master_sig=in_master_sig
+     ,decider_name=in_decider_name
+     ,is_active=in_is_active
+     ,read_only=in_read_only
+     ,last_change=in_last_change
+  WHERE decider_pub=in_decider_pub;
+END $$;
+COMMENT ON FUNCTION exchange_do_insert_aml_officer(BYTEA, BYTEA, VARCHAR, BOOL, BOOL, INT8)
+  IS 'Inserts or updates AML staff record, making sure the update is more recent than the previous change';
 CREATE OR REPLACE FUNCTION exchange_do_batch_reserves_in_insert(
   IN in_reserve_pub BYTEA,
   IN in_expiration_date INT8,
@@ -2112,7 +2265,7 @@ CREATE OR REPLACE FUNCTION exchange_do_batch_reserves_in_insert(
   IN in_credit_val INT8,
   IN in_credit_frac INT4,
   IN in_exchange_account_name VARCHAR,
-  IN in_exectution_date INT8,
+  IN in_execution_date INT8,
   IN in_wire_source_h_payto BYTEA, 
   IN in_payto_uri VARCHAR,
   IN in_reserve_expiration INT8,
@@ -2129,9 +2282,9 @@ DECLARE
 DECLARE
   curs_trans refcursor;
 BEGIN
-ruuid= 0;
-out_reserve_found = TRUE;
-transaction_duplicate= TRUE;
+  ruuid = 0;
+  out_reserve_found = TRUE;
+  transaction_duplicate = TRUE;
   INSERT INTO wire_targets
     (wire_target_h_payto
     ,payto_uri)
@@ -2166,7 +2319,6 @@ transaction_duplicate= TRUE;
     END IF;
   END IF;
   CLOSE curs;
-  PERFORM pg_notify(in_notify, NULL);
   OPEN curs_trans FOR
   WITH reserve_transaction AS(
   INSERT INTO reserves_in
@@ -2184,7 +2336,7 @@ transaction_duplicate= TRUE;
     ,in_credit_frac
     ,in_exchange_account_name
     ,in_wire_source_h_payto
-    ,in_expiration_date)
+    ,in_execution_date)
     ON CONFLICT DO NOTHING
     RETURNING reserve_pub)
   SELECT * FROM reserve_transaction;
@@ -2194,6 +2346,9 @@ transaction_duplicate= TRUE;
     IF i.reserve_pub = in_reserve_pub
     THEN
        transaction_duplicate = FALSE;
+       EXECUTE FORMAT (
+         'NOTIFY %s'
+         ,in_notify);
     END IF;
   END IF;
   CLOSE curs_trans;
@@ -2263,7 +2418,7 @@ CREATE OR REPLACE FUNCTION exchange_do_batch2_reserves_insert(
   IN in_credit_val INT8,
   IN in_credit_frac INT4,
   IN in_exchange_account_name VARCHAR,
-  IN in_exectution_date INT8,
+  IN in_execution_date INT8,
   IN in_wire_source_h_payto BYTEA, 
   IN in_payto_uri VARCHAR,
   IN in_reserve_expiration INT8,
@@ -2274,7 +2429,7 @@ CREATE OR REPLACE FUNCTION exchange_do_batch2_reserves_insert(
   IN in2_credit_val INT8,
   IN in2_credit_frac INT4,
   IN in2_exchange_account_name VARCHAR,
-  IN in2_exectution_date INT8,
+  IN in2_execution_date INT8,
   IN in2_wire_source_h_payto BYTEA, 
   IN in2_payto_uri VARCHAR,
   IN in2_reserve_expiration INT8,
@@ -2375,14 +2530,14 @@ BEGIN
     ,in_credit_frac
     ,in_exchange_account_name
     ,in_wire_source_h_payto
-    ,in_expiration_date),
+    ,in_execution_date),
     (in2_reserve_pub
     ,in2_wire_ref
     ,in2_credit_val
     ,in2_credit_frac
     ,in2_exchange_account_name
     ,in2_wire_source_h_payto
-    ,in_expiration_date)
+    ,in_execution_date)
     ON CONFLICT DO NOTHING
     RETURNING reserve_pub)
   SELECT * FROM reserve_in_exist;
@@ -2421,7 +2576,7 @@ CREATE OR REPLACE FUNCTION exchange_do_batch4_reserves_insert(
   IN in_credit_val INT8,
   IN in_credit_frac INT4,
   IN in_exchange_account_name VARCHAR,
-  IN in_exectution_date INT8,
+  IN in_execution_date INT8,
   IN in_wire_source_h_payto BYTEA, 
   IN in_payto_uri VARCHAR,
   IN in_reserve_expiration INT8,
@@ -2434,7 +2589,7 @@ CREATE OR REPLACE FUNCTION exchange_do_batch4_reserves_insert(
   IN in2_credit_val INT8,
   IN in2_credit_frac INT4,
   IN in2_exchange_account_name VARCHAR,
-  IN in2_exectution_date INT8,
+  IN in2_execution_date INT8,
   IN in2_wire_source_h_payto BYTEA, 
   IN in2_payto_uri VARCHAR,
   IN in2_reserve_expiration INT8,
@@ -2443,7 +2598,7 @@ CREATE OR REPLACE FUNCTION exchange_do_batch4_reserves_insert(
   IN in3_credit_val INT8,
   IN in3_credit_frac INT4,
   IN in3_exchange_account_name VARCHAR,
-  IN in3_exectution_date INT8,
+  IN in3_execution_date INT8,
   IN in3_wire_source_h_payto BYTEA, 
   IN in3_payto_uri VARCHAR,
   IN in3_reserve_expiration INT8,
@@ -2452,7 +2607,7 @@ CREATE OR REPLACE FUNCTION exchange_do_batch4_reserves_insert(
   IN in4_credit_val INT8,
   IN in4_credit_frac INT4,
   IN in4_exchange_account_name VARCHAR,
-  IN in4_exectution_date INT8,
+  IN in4_execution_date INT8,
   IN in4_wire_source_h_payto BYTEA, 
   IN in4_payto_uri VARCHAR,
   IN in4_reserve_expiration INT8,
@@ -2611,28 +2766,28 @@ BEGIN
     ,in_credit_frac
     ,in_exchange_account_name
     ,in_wire_source_h_payto
-    ,in_expiration_date),
+    ,in_execution_date),
     (in2_reserve_pub
     ,in2_wire_ref
     ,in2_credit_val
     ,in2_credit_frac
     ,in2_exchange_account_name
     ,in2_wire_source_h_payto
-    ,in_expiration_date),
+    ,in_execution_date),
     (in3_reserve_pub
     ,in3_wire_ref
     ,in3_credit_val
     ,in3_credit_frac
     ,in3_exchange_account_name
     ,in3_wire_source_h_payto
-    ,in_expiration_date),
+    ,in_execution_date),
     (in4_reserve_pub
     ,in4_wire_ref
     ,in4_credit_val
     ,in4_credit_frac
     ,in4_exchange_account_name
     ,in4_wire_source_h_payto
-    ,in_expiration_date)
+    ,in_execution_date)
      ON CONFLICT DO NOTHING
      RETURNING reserve_pub)
     SELECT * FROM reserve_in_changes;
@@ -2670,7 +2825,7 @@ CREATE OR REPLACE FUNCTION exchange_do_batch8_reserves_insert(
   IN in_credit_val INT8,
   IN in_credit_frac INT4,
   IN in_exchange_account_name VARCHAR,
-  IN in_exectution_date INT8,
+  IN in_execution_date INT8,
   IN in_wire_source_h_payto BYTEA, 
   IN in_payto_uri VARCHAR,
   IN in_reserve_expiration INT8,
@@ -2687,7 +2842,7 @@ CREATE OR REPLACE FUNCTION exchange_do_batch8_reserves_insert(
   IN in2_credit_val INT8,
   IN in2_credit_frac INT4,
   IN in2_exchange_account_name VARCHAR,
-  IN in2_exectution_date INT8,
+  IN in2_execution_date INT8,
   IN in2_wire_source_h_payto BYTEA, 
   IN in2_payto_uri VARCHAR,
   IN in2_reserve_expiration INT8,
@@ -2696,7 +2851,7 @@ CREATE OR REPLACE FUNCTION exchange_do_batch8_reserves_insert(
   IN in3_credit_val INT8,
   IN in3_credit_frac INT4,
   IN in3_exchange_account_name VARCHAR,
-  IN in3_exectution_date INT8,
+  IN in3_execution_date INT8,
   IN in3_wire_source_h_payto BYTEA, 
   IN in3_payto_uri VARCHAR,
   IN in3_reserve_expiration INT8,
@@ -2705,7 +2860,7 @@ CREATE OR REPLACE FUNCTION exchange_do_batch8_reserves_insert(
   IN in4_credit_val INT8,
   IN in4_credit_frac INT4,
   IN in4_exchange_account_name VARCHAR,
-  IN in4_exectution_date INT8,
+  IN in4_execution_date INT8,
   IN in4_wire_source_h_payto BYTEA, 
   IN in4_payto_uri VARCHAR,
   IN in4_reserve_expiration INT8,
@@ -2714,7 +2869,7 @@ CREATE OR REPLACE FUNCTION exchange_do_batch8_reserves_insert(
   IN in5_credit_val INT8,
   IN in5_credit_frac INT4,
   IN in5_exchange_account_name VARCHAR,
-  IN in5_exectution_date INT8,
+  IN in5_execution_date INT8,
   IN in5_wire_source_h_payto BYTEA, 
   IN in5_payto_uri VARCHAR,
   IN in5_reserve_expiration INT8,
@@ -2723,7 +2878,7 @@ CREATE OR REPLACE FUNCTION exchange_do_batch8_reserves_insert(
   IN in6_credit_val INT8,
   IN in6_credit_frac INT4,
   IN in6_exchange_account_name VARCHAR,
-  IN in6_exectution_date INT8,
+  IN in6_execution_date INT8,
   IN in6_wire_source_h_payto BYTEA, 
   IN in6_payto_uri VARCHAR,
   IN in6_reserve_expiration INT8,
@@ -2732,7 +2887,7 @@ CREATE OR REPLACE FUNCTION exchange_do_batch8_reserves_insert(
   IN in7_credit_val INT8,
   IN in7_credit_frac INT4,
   IN in7_exchange_account_name VARCHAR,
-  IN in7_exectution_date INT8,
+  IN in7_execution_date INT8,
   IN in7_wire_source_h_payto BYTEA, 
   IN in7_payto_uri VARCHAR,
   IN in7_reserve_expiration INT8,
@@ -2741,7 +2896,7 @@ CREATE OR REPLACE FUNCTION exchange_do_batch8_reserves_insert(
   IN in8_credit_val INT8,
   IN in8_credit_frac INT4,
   IN in8_exchange_account_name VARCHAR,
-  IN in8_exectution_date INT8,
+  IN in8_execution_date INT8,
   IN in8_wire_source_h_payto BYTEA, 
   IN in8_payto_uri VARCHAR,
   IN in8_reserve_expiration INT8,
@@ -3034,56 +3189,56 @@ BEGIN
     ,in_credit_frac
     ,in_exchange_account_name
     ,in_wire_source_h_payto
-    ,in_expiration_date),
+    ,in_execution_date),
     (in2_reserve_pub
     ,in2_wire_ref
     ,in2_credit_val
     ,in2_credit_frac
     ,in2_exchange_account_name
     ,in2_wire_source_h_payto
-    ,in_expiration_date),
+    ,in_execution_date),
     (in3_reserve_pub
     ,in3_wire_ref
     ,in3_credit_val
     ,in3_credit_frac
     ,in3_exchange_account_name
     ,in3_wire_source_h_payto
-    ,in_expiration_date),
+    ,in_execution_date),
     (in4_reserve_pub
     ,in4_wire_ref
     ,in4_credit_val
     ,in4_credit_frac
     ,in4_exchange_account_name
     ,in4_wire_source_h_payto
-    ,in_expiration_date),
+    ,in_execution_date),
     (in5_reserve_pub
     ,in5_wire_ref
     ,in5_credit_val
     ,in5_credit_frac
     ,in5_exchange_account_name
     ,in5_wire_source_h_payto
-    ,in_expiration_date),
+    ,in_execution_date),
     (in6_reserve_pub
     ,in6_wire_ref
     ,in6_credit_val
     ,in6_credit_frac
     ,in6_exchange_account_name
     ,in6_wire_source_h_payto
-    ,in_expiration_date),
+    ,in_execution_date),
     (in7_reserve_pub
     ,in7_wire_ref
     ,in7_credit_val
     ,in7_credit_frac
     ,in7_exchange_account_name
     ,in7_wire_source_h_payto
-    ,in_expiration_date),
+    ,in_execution_date),
     (in8_reserve_pub
     ,in8_wire_ref
     ,in8_credit_val
     ,in8_credit_frac
     ,in8_exchange_account_name
     ,in8_wire_source_h_payto
-    ,in_expiration_date)
+    ,in_execution_date)
      ON CONFLICT DO NOTHING
      RETURNING reserve_pub)
     SELECT * FROM reserve_in_changes;
@@ -3129,4 +3284,135 @@ BEGIN
   CLOSE curs_transaction_existed;
   RETURN;
 END $$;
+CREATE OR REPLACE FUNCTION exchange_do_refund_by_coin(
+  IN in_coin_pub BYTEA,
+  IN in_merchant_pub BYTEA,
+  IN in_h_contract BYTEA
+)
+RETURNS SETOF record
+LANGUAGE plpgsql
+AS $$
+DECLARE
+  curs CURSOR
+  FOR
+  SELECT
+    amount_with_fee_val
+   ,amount_with_fee_frac
+   ,deposit_serial_id
+  FROM refunds
+  WHERE coin_pub=in_coin_pub;
+DECLARE
+  i RECORD;
+BEGIN
+OPEN curs;
+LOOP
+    FETCH NEXT FROM curs INTO i;
+    EXIT WHEN NOT FOUND;
+    RETURN QUERY
+      SELECT
+        i.amount_with_fee_val
+       ,i.amount_with_fee_frac
+       FROM deposits
+       WHERE
+         coin_pub=in_coin_pub
+         AND merchant_pub=in_merchant_pub
+         AND h_contract_terms=in_h_contract
+         AND i.deposit_serial_id = deposit_serial_id;
+END LOOP;
+CLOSE curs;
+END $$;
+CREATE OR REPLACE FUNCTION exchange_do_get_ready_deposit(
+  IN in_now INT8,
+  IN in_start_shard_now INT8,
+  IN in_end_shard_now INT8,
+  OUT out_payto_uri VARCHAR,
+  OUT out_merchant_pub BYTEA
+)
+LANGUAGE plpgsql
+AS $$
+DECLARE
+ var_wire_target_h_payto BYTEA;
+DECLARE
+ var_coin_pub BYTEA;
+DECLARE
+ var_deposit_serial_id INT8;
+DECLARE
+ curs CURSOR
+ FOR
+ SELECT
+   coin_pub
+  ,deposit_serial_id
+  ,wire_deadline
+  ,shard
+ FROM deposits_by_ready
+ WHERE wire_deadline <= in_now
+ AND shard >=in_start_shard_now
+ AND shard <=in_end_shard_now
+ LIMIT 1;
+DECLARE
+ i RECORD;
+BEGIN
+OPEN curs;
+FETCH FROM curs INTO i;
+IF NOT FOUND
+THEN
+  RETURN;
+END IF;
+SELECT
+   payto_uri
+  ,merchant_pub
+  INTO
+   out_payto_uri
+  ,out_merchant_pub
+  FROM deposits dep
+  JOIN wire_targets wt
+  ON (wt.wire_target_h_payto=dep.wire_target_h_payto)
+  WHERE dep.coin_pub=i.coin_pub
+  AND dep.deposit_serial_id=i.deposit_serial_id
+  ORDER BY
+   i.wire_deadline ASC
+  ,i.shard ASC;
+RETURN;
+END $$;
+CREATE OR REPLACE FUNCTION exchange_do_get_link_data(
+  IN in_coin_pub BYTEA
+)
+RETURNS SETOF record
+LANGUAGE plpgsql
+AS $$
+DECLARE
+  curs CURSOR
+  FOR
+  SELECT
+   melt_serial_id
+  FROM refresh_commitments
+  WHERE old_coin_pub=in_coin_pub;
+DECLARE
+  i RECORD;
+BEGIN
+OPEN curs;
+LOOP
+    FETCH NEXT FROM curs INTO i;
+    EXIT WHEN NOT FOUND;
+    RETURN QUERY
+      SELECT
+       tp.transfer_pub
+      ,denoms.denom_pub
+      ,rrc.ev_sig
+      ,rrc.ewv
+      ,rrc.link_sig
+      ,rrc.freshcoin_index
+      ,rrc.coin_ev
+      FROM refresh_revealed_coins rrc
+       JOIN refresh_transfer_keys tp
+         ON (tp.melt_serial_id=rrc.melt_serial_id)
+       JOIN denominations denoms
+         ON (rrc.denominations_serial=denoms.denominations_serial)
+       WHERE rrc.melt_serial_id =i.melt_serial_id
+       ORDER BY tp.transfer_pub,
+       rrc.freshcoin_index ASC
+       ;
+END LOOP;
+CLOSE curs;
+END $$;
 COMMIT;
