@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2022 Taler Systems SA
+  Copyright (C) 2014-2023 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -980,6 +980,7 @@ test_wire_out (const struct TALER_EXCHANGEDB_Deposit *deposit)
     struct TALER_Amount coin_fee2;
     struct GNUNET_TIME_Timestamp execution_time2;
     struct TALER_EXCHANGEDB_KycStatus kyc;
+    enum TALER_AmlDecisionState aml;
 
     h_contract_terms_wt2.hash.bits[0]++;
     FAILIF (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS !=
@@ -993,7 +994,8 @@ test_wire_out (const struct TALER_EXCHANGEDB_Deposit *deposit)
                                                 &execution_time2,
                                                 &coin_contribution2,
                                                 &coin_fee2,
-                                                &kyc));
+                                                &kyc,
+                                                &aml));
   }
   {
     struct TALER_ReservePublicKeyP rpub;
@@ -1025,6 +1027,7 @@ test_wire_out (const struct TALER_EXCHANGEDB_Deposit *deposit)
     struct TALER_Amount coin_fee2;
     struct GNUNET_TIME_Timestamp execution_time2;
     struct TALER_EXCHANGEDB_KycStatus kyc;
+    enum TALER_AmlDecisionState aml = TALER_AML_FROZEN;
 
     FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
             plugin->lookup_transfer_by_deposit (plugin->cls,
@@ -1037,7 +1040,9 @@ test_wire_out (const struct TALER_EXCHANGEDB_Deposit *deposit)
                                                 &execution_time2,
                                                 &coin_contribution2,
                                                 &coin_fee2,
-                                                &kyc));
+                                                &kyc,
+                                                &aml));
+    FAILIF (TALER_AML_NORMAL != aml);
     GNUNET_assert (0 == GNUNET_memcmp (&wtid2,
                                        &wire_out_wtid));
     GNUNET_assert (GNUNET_TIME_timestamp_cmp (execution_time2,
@@ -1288,16 +1293,29 @@ run (void *cls)
                  TALER_string_to_amount (CURRENCY ":1.000010",
                                          &amount_with_fee));
   result = 4;
+  FAILIF (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS !=
+          plugin->commit (plugin->cls));
   now = GNUNET_TIME_timestamp_get ();
-  FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
-          plugin->reserves_in_insert (plugin->cls,
-                                      &reserve_pub,
-                                      &value,
-                                      now,
-                                      sndr,
-                                      "exchange-account-1",
-                                      4));
+  {
+    struct TALER_EXCHANGEDB_ReserveInInfo reserve = {
+      .reserve_pub = &reserve_pub,
+      .balance = &value,
+      .execution_time = now,
+      .sender_account_details = sndr,
+      .exchange_account_name = "exchange-account-1",
+      .wire_reference = 4
+    };
+    enum GNUNET_DB_QueryStatus qsr;
 
+    FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
+            plugin->reserves_in_insert (plugin->cls,
+                                        &reserve,
+                                        1,
+                                        1,
+                                        &qsr));
+    FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
+            qsr);
+  }
   FAILIF (GNUNET_OK !=
           check_reserve (&reserve_pub,
                          value.value,
@@ -1305,14 +1323,29 @@ run (void *cls)
                          value.currency));
   now = GNUNET_TIME_timestamp_get ();
   RND_BLK (&reserve_pub2);
-  FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
-          plugin->reserves_in_insert (plugin->cls,
-                                      &reserve_pub2,
-                                      &value,
-                                      now,
-                                      sndr,
-                                      "exchange-account-1",
-                                      5));
+  {
+    struct TALER_EXCHANGEDB_ReserveInInfo reserve = {
+      .reserve_pub = &reserve_pub2,
+      .balance = &value,
+      .execution_time = now,
+      .sender_account_details = sndr,
+      .exchange_account_name = "exchange-account-1",
+      .wire_reference = 5
+    };
+    enum GNUNET_DB_QueryStatus qsr;
+
+    FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
+            plugin->reserves_in_insert (plugin->cls,
+                                        &reserve,
+                                        1,
+                                        1,
+                                        &qsr));
+    FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
+            qsr);
+  }
+  FAILIF (GNUNET_OK !=
+          plugin->start (plugin->cls,
+                         "test-2"));
   FAILIF (GNUNET_OK !=
           check_reserve (&reserve_pub,
                          value.value,
