@@ -214,36 +214,41 @@ typedef enum GNUNET_DB_QueryStatus
  *         amounts involved in this type of operation
  *         at the given account
  * @param ai_cls closure for @a ai
- * @return NULL if no check is needed,
+ * @param[out] required set to NULL if no check is needed,
  *   otherwise space-separated list of required checks
+ * @return transaction status
  */
-const char *
+enum GNUNET_DB_QueryStatus
 TALER_KYCLOGIC_kyc_test_required (enum TALER_KYCLOGIC_KycTriggerEvent event,
                                   const struct TALER_PaytoHashP *h_payto,
                                   TALER_KYCLOGIC_KycSatisfiedIterator ki,
                                   void *ki_cls,
                                   TALER_KYCLOGIC_KycAmountIterator ai,
-                                  void *ai_cls);
+                                  void *ai_cls,
+                                  char **required);
 
 
 /**
  * Check if the @a requirements are now satsified for
  * @a h_payto account.
  *
- * @param requirements space-spearated list of requirements
+ * @param[in,out] requirements space-spearated list of requirements,
+ *       already satisfied requirements are removed from the list
  * @param h_payto hash over the account
  * @param[out] kyc_details if satisfied, set to what kind of
  *             KYC information was collected
  * @param ki iterator over satisfied providers
  * @param ki_cls closure for @a ki
- * @return true if the KYC check was satisfied
+ * @param[out] satisfied set to true if the KYC check was satisfied
+ * @return transaction status (from @a ki)
  */
-bool
-TALER_KYCLOGIC_check_satisfied (const char *requirements,
+enum GNUNET_DB_QueryStatus
+TALER_KYCLOGIC_check_satisfied (char **requirements,
                                 const struct TALER_PaytoHashP *h_payto,
                                 json_t **kyc_details,
                                 TALER_KYCLOGIC_KycSatisfiedIterator ki,
-                                void *ki_cls);
+                                void *ki_cls,
+                                bool *satisfied);
 
 
 /**
@@ -292,6 +297,29 @@ TALER_KYCLOGIC_kyc_get_details (
   void *cb_cls);
 
 
+/**
+ * Check if a given @a check_name is a legal name (properly
+ * configured) and can be satisfied in principle.
+ *
+ * @param check_name name of the check to see if it is configured
+ * @return #GNUNET_OK if the check can be satisfied,
+ *         #GNUNET_NO if the check can never be satisfied,
+ *         #GNUNET_SYSERR if the type of the check is unknown
+ */
+enum GNUNET_GenericReturnValue
+TALER_KYCLOGIC_check_satisfiable (
+  const char *check_name);
+
+
+/**
+ * Return list of all KYC checks that are possible.
+ *
+ * @return JSON array of strings with the allowed KYC checks
+ */
+json_t *
+TALER_KYCLOGIC_get_satisfiable (void);
+
+
 /**
  * Obtain the provider logic for a given set of @a requirements.
  *
@@ -309,27 +337,6 @@ TALER_KYCLOGIC_requirements_to_logic (const char *requirements,
                                       const char **configuration_section);
 
 
-/**
- * Obtain attributes we collected about a user from a
- * provider.
- *
- * @param provider_section configuration section of a
- *    provider that triggered KYC process for a user
- * @param provider_user_id user ID of the user at the provider
- * @param legitimization_id legitimizatin ID of a process
- *    of that user at the provider
- * @param[out] attr_expiration set to when the @a attrs expire
- * @param[out] attrs attributes we have about the user
- * @return error code, #TALER_EC_NONE on success
- */
-enum TALER_ErrorCode
-TALER_KYCLOGIC_user_to_attributes (const char *provider_section,
-                                   const char *provider_user_id,
-                                   const char *legitimization_id,
-                                   struct GNUNET_TIME_Timestamp *attr_expiration,
-                                   json_t **attrs);
-
-
 /**
  * Obtain the provider logic for a given @a name.
  *
