@@ -229,6 +229,12 @@
 #define TALER_SIGNATURE_EXCHANGE_CONFIRM_PURSE_REFUND 1049
 
 
+/**
+ * Signature where the Exchange confirms an age-withdraw.
+ */
+#define TALER_SIGNATURE_EXCHANGE_CONFIRM_AGE_WITHDRAW 1050
+
+
 /**
  * Signature where the auditor confirms that he is aware of certain denomination keys from the exchange.
  */
@@ -391,6 +397,12 @@
 #define TALER_SIGNATURE_WALLET_PURSE_DELETE 1220
 
 
+/**
+ * Signature where the reserve key confirms an age-withdraw request. Signed with the reserve private key.
+ */
+#define TALER_SIGNATURE_WALLET_RESERVE_AGE_WITHDRAW 1221
+
+
 /**
  * Signature on a denomination key announcement.
  */
@@ -427,6 +439,12 @@
 #define TALER_SIGNATURE_AML_DECISION 1350
 
 
+/**
+ * Signature by which an AML officer requests AML data.
+ */
+#define TALER_SIGNATURE_AML_QUERY 1351
+
+
 /**
  * EdDSA signature for a policy upload.
  */
