@@ -21,6 +21,12 @@
 #include "platform.h"
 #include "taler_kyclogic_lib.h"
 
+/**
+ * Name of the KYC check that may never be passed. Useful if some
+ * operations/amounts are categorically forbidden.
+ */
+#define KYC_CHECK_IMPOSSIBLE "impossible"
+
 /**
  * Information about a KYC provider.
  */
@@ -265,6 +271,38 @@ TALER_KYCLOGIC_kyc_user_type2s (enum TALER_KYCLOGIC_KycUserType ut)
 }
 
 
+enum GNUNET_GenericReturnValue
+TALER_KYCLOGIC_check_satisfiable (
+  const char *check_name)
+{
+  for (unsigned int i = 0; i<num_kyc_checks; i++)
+    if (0 == strcmp (check_name,
+                     kyc_checks[i]->name))
+      return GNUNET_OK;
+  if (0 == strcmp (check_name,
+                   KYC_CHECK_IMPOSSIBLE))
+    return GNUNET_NO;
+  return GNUNET_SYSERR;
+}
+
+
+json_t *
+TALER_KYCLOGIC_get_satisfiable ()
+{
+  json_t *requirements;
+
+  requirements = json_array ();
+  GNUNET_assert (NULL != requirements);
+  for (unsigned int i = 0; i<num_kyc_checks; i++)
+    GNUNET_assert (
+      0 ==
+      json_array_append_new (
+        requirements,
+        json_string (kyc_checks[i]->name)));
+  return requirements;
+}
+
+
 /**
  * Load KYC logic plugin.
  *
@@ -331,9 +369,8 @@ add_check (const char *check)
 
 
 /**
- * Parse list of checks from @a checks and build an
- * array of aliases into the global checks array
- * in @a provided_checks.
+ * Parse list of checks from @a checks and build an array of aliases into the
+ * global checks array in @a provided_checks.
  *
  * @param[in,out] checks list of checks; clobbered
  * @param[out] p_checks where to put array of aliases
@@ -585,6 +622,29 @@ add_trigger (const struct GNUNET_CONFIGURATION_Handle *cfg,
     GNUNET_array_append (kyc_triggers,
                          num_kyc_triggers,
                          kt);
+    for (unsigned int i = 0; i<kt->num_checks; i++)
+    {
+      const struct TALER_KYCLOGIC_KycCheck *ck = kt->required_checks[i];
+
+      if (0 != ck->num_providers)
+        continue;
+      if (0 == strcmp (ck->name,
+                       KYC_CHECK_IMPOSSIBLE))
+        continue;
+      {
+        char *msg;
+
+        GNUNET_asprintf (&msg,
+                         "Required check `%s' cannot be satisfied: not provided by any provider",
+                         ck->name);
+        GNUNET_log_config_invalid (GNUNET_ERROR_TYPE_ERROR,
+                                   section,
+                                   "REQUIRED_CHECKS",
+                                   msg);
+        GNUNET_free (msg);
+      }
+      return GNUNET_SYSERR;
+    }
   }
   return GNUNET_OK;
 }
@@ -614,8 +674,8 @@ struct SectionContext
  * @param section name of the section
  */
 static void
-handle_section (void *cls,
-                const char *section)
+handle_provider_section (void *cls,
+                         const char *section)
 {
   struct SectionContext *sc = cls;
 
@@ -629,6 +689,21 @@ handle_section (void *cls,
       sc->result = false;
     return;
   }
+}
+
+
+/**
+ * Function to iterate over configuration sections.
+ *
+ * @param cls a `struct SectionContext *`
+ * @param section name of the section
+ */
+static void
+handle_trigger_section (void *cls,
+                        const char *section)
+{
+  struct SectionContext *sc = cls;
+
   if (0 == strncasecmp (section,
                         "kyc-legitimization-",
                         strlen ("kyc-legitimization-")))
@@ -680,7 +755,10 @@ TALER_KYCLOGIC_kyc_init (const struct GNUNET_CONFIGURATION_Handle *cfg)
   };
 
   GNUNET_CONFIGURATION_iterate_sections (cfg,
-                                         &handle_section,
+                                         &handle_provider_section,
+                                         &sc);
+  GNUNET_CONFIGURATION_iterate_sections (cfg,
+                                         &handle_trigger_section,
                                          &sc);
   if (! sc.result)
   {
@@ -996,13 +1074,14 @@ remove_satisfied (void *cls,
 }
 
 
-const char *
+enum GNUNET_DB_QueryStatus
 TALER_KYCLOGIC_kyc_test_required (enum TALER_KYCLOGIC_KycTriggerEvent event,
                                   const struct TALER_PaytoHashP *h_payto,
                                   TALER_KYCLOGIC_KycSatisfiedIterator ki,
                                   void *ki_cls,
                                   TALER_KYCLOGIC_KycAmountIterator ai,
-                                  void *ai_cls)
+                                  void *ai_cls,
+                                  char **required)
 {
   struct TALER_KYCLOGIC_KycCheck *needed[num_kyc_checks];
   unsigned int needed_cnt = 0;
@@ -1035,7 +1114,10 @@ TALER_KYCLOGIC_kyc_test_required (enum TALER_KYCLOGIC_KycTriggerEvent event,
         &ttc);
   }
   if (0 == needed_cnt)
-    return NULL;
+  {
+    *required = NULL;
+    return GNUNET_DB_STATUS_SUCCESS_NO_RESULTS;
+  }
   timeframe = GNUNET_TIME_UNIT_ZERO;
   for (unsigned int i = 0; i<num_kyc_triggers; i++)
   {
@@ -1062,7 +1144,10 @@ TALER_KYCLOGIC_kyc_test_required (enum TALER_KYCLOGIC_KycTriggerEvent event,
         &ttc);
   }
   if (0 == needed_cnt)
-    return NULL;
+  {
+    *required = NULL;
+    return GNUNET_DB_STATUS_SUCCESS_NO_RESULTS;
+  }
   {
     struct RemoveContext rc = {
       .needed = needed,
@@ -1076,10 +1161,17 @@ TALER_KYCLOGIC_kyc_test_required (enum TALER_KYCLOGIC_KycTriggerEvent event,
              h_payto,
              &remove_satisfied,
              &rc);
-    GNUNET_break (qs >= 0);  // FIXME: handle DB failure more nicely?
+    if (qs < 0)
+    {
+      GNUNET_break (GNUNET_DB_STATUS_SOFT_ERROR == qs);
+      return qs;
+    }
   }
   if (0 == needed_cnt)
-    return NULL;
+  {
+    *required = NULL;
+    return GNUNET_DB_STATUS_SUCCESS_NO_RESULTS;
+  }
   {
     struct RemoveContext rc = {
       .needed = needed,
@@ -1093,10 +1185,17 @@ TALER_KYCLOGIC_kyc_test_required (enum TALER_KYCLOGIC_KycTriggerEvent event,
              h_payto,
              &remove_satisfied,
              &rc);
-    GNUNET_break (qs >= 0);  // FIXME: handle DB failure more nicely?
+    if (qs < 0)
+    {
+      GNUNET_break (GNUNET_DB_STATUS_SOFT_ERROR == qs);
+      return qs;
+    }
   }
   if (0 == needed_cnt)
-    return NULL;
+  {
+    *required = NULL;
+    return GNUNET_DB_STATUS_SUCCESS_NO_RESULTS;
+  }
   ret = NULL;
   for (unsigned int k = 0; k<needed_cnt; k++)
   {
@@ -1117,7 +1216,8 @@ TALER_KYCLOGIC_kyc_test_required (enum TALER_KYCLOGIC_KycTriggerEvent event,
       GNUNET_free (tmp);
     }
   }
-  return ret;
+  *required = ret;
+  return GNUNET_DB_STATUS_SUCCESS_ONE_RESULT;
 }
 
 
@@ -1144,26 +1244,31 @@ TALER_KYCLOGIC_kyc_get_details (
 }
 
 
-bool
-TALER_KYCLOGIC_check_satisfied (const char *requirements,
+enum GNUNET_DB_QueryStatus
+TALER_KYCLOGIC_check_satisfied (char **requirements,
                                 const struct TALER_PaytoHashP *h_payto,
                                 json_t **kyc_details,
                                 TALER_KYCLOGIC_KycSatisfiedIterator ki,
-                                void *ki_cls)
+                                void *ki_cls,
+                                bool *satisfied)
 {
   struct TALER_KYCLOGIC_KycCheck *needed[num_kyc_checks];
   unsigned int needed_cnt = 0;
 
   if (NULL == requirements)
-    return true;
   {
-    char *req = GNUNET_strdup (requirements);
+    *satisfied = true;
+    return GNUNET_DB_STATUS_SUCCESS_NO_RESULTS;
+  }
+  {
+    char *req = *requirements;
 
     for (const char *tok = strtok (req, " ");
          NULL != tok;
          tok = strtok (NULL, " "))
       needed[needed_cnt++] = add_check (tok);
     GNUNET_free (req);
+    *requirements = NULL;
   }
 
   {
@@ -1182,7 +1287,12 @@ TALER_KYCLOGIC_check_satisfied (const char *requirements,
              h_payto,
              &remove_satisfied,
              &rc);
-    GNUNET_break (qs >= 0);  // FIXME: handle DB failure more nicely?
+    if (qs < 0)
+    {
+      GNUNET_break (GNUNET_DB_STATUS_SOFT_ERROR == qs);
+      *satisfied = false;
+      return qs;
+    }
     if (0 != needed_cnt)
     {
       json_decref (rc.kyc_details);
@@ -1193,7 +1303,34 @@ TALER_KYCLOGIC_check_satisfied (const char *requirements,
       *kyc_details = rc.kyc_details;
     }
   }
-  return (0 == needed_cnt);
+  *satisfied = (0 == needed_cnt);
+
+  {
+    char *res = NULL;
+
+    for (unsigned int i = 0; i<needed_cnt; i++)
+    {
+      const struct TALER_KYCLOGIC_KycCheck *need = needed[i];
+
+      if (NULL == res)
+      {
+        res = GNUNET_strdup (need->name);
+      }
+      else
+      {
+        char *tmp;
+
+        GNUNET_asprintf (&tmp,
+                         "%s %s",
+                         res,
+                         need->name);
+        GNUNET_free (res);
+        res = tmp;
+      }
+    }
+    *requirements = res;
+  }
+  return GNUNET_DB_STATUS_SUCCESS_ONE_RESULT;
 }
 
 
@@ -1339,18 +1476,4 @@ TALER_KYCLOGIC_kyc_iterate_thresholds (
 }
 
 
-enum TALER_ErrorCode
-TALER_KYCLOGIC_user_to_attributes (const char *provider_section,
-                                   const char *provider_user_id,
-                                   const char *legitimization_id,
-                                   struct GNUNET_TIME_Timestamp *attr_expiration,
-                                   json_t **attrs)
-{
-  GNUNET_break (0); // FIXME: not yet implemented!!!
-  *attrs = json_object ();
-  *attr_expiration = GNUNET_TIME_UNIT_ZERO_TS;
-  return TALER_EC_NONE;
-}
-
-
 /* end of taler-exchange-httpd_kyc.c */
