@@ -1,6 +1,6 @@
 /*
   This file is part of GNU Taler
-  Copyright (C) 2022 Taler Systems SA
+  Copyright (C) 2022, 2023 Taler Systems SA
 
   Taler is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -19,6 +19,7 @@
  * @author Christian Grothoff
  */
 #include "platform.h"
+#include "taler_attributes.h"
 #include "taler_kyclogic_plugin.h"
 #include "taler_mhd_lib.h"
 #include "taler_curl_lib.h"
@@ -62,9 +63,10 @@ struct PluginState
   struct GNUNET_CURL_RescheduleContext *curl_rc;
 
   /**
- * Authorization token to use when receiving webhooks from the Persona service.  Optional.  Note that
- * webhooks are *global* and not per template.
- */
+   * Authorization token to use when receiving webhooks from the Persona
+   * service.  Optional.  Note that webhooks are *global* and not per
+   * template.
+   */
   char *webhook_token;
 
 
@@ -866,12 +868,9 @@ proof_generic_reply (struct TALER_KYCLOGIC_ProofHandle *ph,
 {
   struct MHD_Response *resp;
   enum GNUNET_GenericReturnValue ret;
-  struct GNUNET_TIME_Absolute expiration;
 
-  if (TALER_KYCLOGIC_STATUS_SUCCESS == status)
-    expiration = GNUNET_TIME_relative_to_absolute (ph->pd->validity);
-  else
-    expiration = GNUNET_TIME_UNIT_ZERO_ABS;
+  /* This API is not usable for successful replies */
+  GNUNET_assert (TALER_KYCLOGIC_STATUS_SUCCESS != status);
   ret = TALER_TEMPLATING_build (ph->connection,
                                 &http_status,
                                 template,
@@ -889,7 +888,8 @@ proof_generic_reply (struct TALER_KYCLOGIC_ProofHandle *ph,
           status,
           account_id,
           inquiry_id,
-          expiration,
+          GNUNET_TIME_UNIT_ZERO_ABS,
+          NULL,
           http_status,
           resp);
 }
@@ -922,9 +922,202 @@ proof_reply_error (struct TALER_KYCLOGIC_ProofHandle *ph,
 }
 
 
+/**
+ * Convert KYC attribute data from Persona response.
+ *
+ * @param attr json array with Persona attribute data
+ * @return KYC attribute data
+ */
+static json_t *
+convert_attributes (const json_t *attr)
+{
+  const char *country_code = NULL;
+  const char *name_first = NULL;
+  const char *name_middle = NULL;
+  const char *name_last = NULL;
+  const char *address_street_1 = NULL;
+  const char *address_street_2 = NULL;
+  const char *address_city = NULL;
+  const char *address_postal_code = NULL;
+  const char *birthdate = NULL;
+  struct GNUNET_JSON_Specification spec[] = {
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_string ("country-code",
+                               &country_code),
+      NULL),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_string ("name-first",
+                               &name_first),
+      NULL),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_string ("name-middle",
+                               &name_middle),
+      NULL),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_string ("name-last",
+                               &name_last),
+      NULL),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_string ("address-street-1",
+                               &address_street_1),
+      NULL),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_string ("address-street-2",
+                               &address_street_2),
+      NULL),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_string ("address-city",
+                               &address_city),
+      NULL),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_string ("address-postal-code",
+                               &address_postal_code),
+      NULL),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_string ("birthdate",
+                               &birthdate),
+      NULL),
+    GNUNET_JSON_spec_end ()
+  };
+  json_t *ret;
+
+  if (GNUNET_OK !=
+      GNUNET_JSON_parse (attr,
+                         spec,
+                         NULL, NULL))
+  {
+    GNUNET_break (0);
+    json_dumpf (attr,
+                stderr,
+                JSON_INDENT (2));
+    return NULL;
+  }
+  {
+    char *name = NULL;
+    char *street = NULL;
+    char *city = NULL;
+
+    if ( (NULL != name_last) ||
+         (NULL != name_first) ||
+         (NULL != name_middle) )
+    {
+      GNUNET_asprintf (&name,
+                       "%s, %s %s",
+                       (NULL != name_last)
+                       ? name_last
+                       : "",
+                       (NULL != name_first)
+                       ? name_first
+                       : "",
+                       (NULL != name_middle)
+                       ? name_middle
+                       : "");
+    }
+    if ( (NULL != address_city) ||
+         (NULL != address_postal_code) )
+    {
+      GNUNET_asprintf (&city,
+                       "%s%s%s %s",
+                       (NULL != country_code)
+                       ? country_code
+                       : "",
+                       (NULL != country_code)
+                       ? "-"
+                       : "",
+                       (NULL != address_postal_code)
+                       ? address_postal_code
+                       : "",
+                       (NULL != address_city)
+                       ? address_city
+                       : "");
+    }
+    if ( (NULL != address_street_1) ||
+         (NULL != address_street_2) )
+    {
+      GNUNET_asprintf (&street,
+                       "%s%s%s",
+                       (NULL != address_street_1)
+                       ? address_street_1
+                       : "",
+                       ( (NULL != address_street_1) &&
+                         (NULL != address_street_2) )
+                       ? "\n"
+                       : "",
+                       (NULL != address_street_2)
+                       ? address_street_2
+                       : "");
+    }
+    ret = GNUNET_JSON_PACK (
+      GNUNET_JSON_pack_allow_null (
+        GNUNET_JSON_pack_string (
+          TALER_ATTRIBUTE_BIRTHDATE,
+          birthdate)),
+      GNUNET_JSON_pack_allow_null (
+        GNUNET_JSON_pack_string (
+          TALER_ATTRIBUTE_FULL_NAME,
+          name)),
+      GNUNET_JSON_pack_allow_null (
+        GNUNET_JSON_pack_string (
+          TALER_ATTRIBUTE_ADDRESS_STREET,
+          street)),
+      GNUNET_JSON_pack_allow_null (
+        GNUNET_JSON_pack_string (
+          TALER_ATTRIBUTE_ADDRESS_CITY,
+          city)),
+      GNUNET_JSON_pack_allow_null (
+        GNUNET_JSON_pack_string (
+          TALER_ATTRIBUTE_RESIDENCES,
+          country_code))
+      );
+    GNUNET_free (street);
+    GNUNET_free (city);
+    GNUNET_free (name);
+  }
+  return ret;
+}
+
+
+/**
+ * Return a response for the @a ph request indicating a
+ * protocol violation by the Persona server.
+ *
+ * @param[in,out] ph request we are processing
+ * @param response_code HTTP status returned by Persona
+ * @param inquiry_id ID of the inquiry this is about
+ * @param detail where the response was wrong
+ * @param data full response data to output
+ */
+static void
+return_invalid_response (struct TALER_KYCLOGIC_ProofHandle *ph,
+                         unsigned int response_code,
+                         const char *inquiry_id,
+                         const char *detail,
+                         const json_t *data)
+{
+  proof_reply_error (
+    ph,
+    inquiry_id,
+    MHD_HTTP_BAD_GATEWAY,
+    "persona-invalid-response",
+    GNUNET_JSON_PACK (
+      GNUNET_JSON_pack_uint64 ("persona_http_status",
+                               response_code),
+      GNUNET_JSON_pack_string ("persona_inquiry_id",
+                               inquiry_id),
+      TALER_JSON_pack_ec (
+        TALER_EC_EXCHANGE_KYC_GENERIC_PROVIDER_UNEXPECTED_REPLY),
+      GNUNET_JSON_pack_string ("detail",
+                               detail),
+      GNUNET_JSON_pack_allow_null (
+        GNUNET_JSON_pack_object_incref ("data",
+                                        (json_t *)
+                                        data))));
+}
+
+
 /**
  * Function called when we're done processing the
- * HTTP "/api/v1/verifications/{verification-id}" request.
+ * HTTP "/api/v1/inquiries/{inquiry-id}" request.
  *
  * @param cls the `struct TALER_KYCLOGIC_InitiateHandle`
  * @param response_code HTTP response code, 0 on error
@@ -949,6 +1142,7 @@ handle_proof_finished (void *cls,
       const char *account_id;
       const char *type = NULL;
       json_t *attributes;
+      json_t *relationships;
       struct GNUNET_JSON_Specification spec[] = {
         GNUNET_JSON_spec_string ("type",
                                  &type),
@@ -956,6 +1150,8 @@ handle_proof_finished (void *cls,
                                  &inquiry_id),
         GNUNET_JSON_spec_json ("attributes",
                                &attributes),
+        GNUNET_JSON_spec_json ("relationships",
+                               &relationships),
         GNUNET_JSON_spec_end ()
       };
 
@@ -968,25 +1164,12 @@ handle_proof_finished (void *cls,
                          "inquiry")) )
       {
         GNUNET_break_op (0);
-        json_dumpf (j,
-                    stderr,
-                    JSON_INDENT (2));
-        proof_reply_error (ph,
-                           inquiry_id,
-                           MHD_HTTP_BAD_GATEWAY,
-                           "persona-logic-failure",
-                           GNUNET_JSON_PACK (
-                             GNUNET_JSON_pack_uint64 ("persona_http_status",
-                                                      response_code),
-                             GNUNET_JSON_pack_string ("persona_inquiry_id",
-                                                      inquiry_id),
-                             TALER_JSON_pack_ec (
-                               TALER_EC_EXCHANGE_KYC_GENERIC_PROVIDER_UNEXPECTED_REPLY),
-                             GNUNET_JSON_pack_string ("detail",
-                                                      "data"),
-                             GNUNET_JSON_pack_allow_null (
-                               GNUNET_JSON_pack_object_incref ("data",
-                                                               (json_t *) data))));
+        return_invalid_response (ph,
+                                 response_code,
+                                 inquiry_id,
+                                 "data",
+                                 data);
+        GNUNET_JSON_parse_free (spec);
         break;
       }
 
@@ -997,10 +1180,10 @@ handle_proof_finished (void *cls,
         struct GNUNET_JSON_Specification ispec[] = {
           GNUNET_JSON_spec_string ("status",
                                    &status),
-          GNUNET_JSON_spec_string ("reference_id",
+          GNUNET_JSON_spec_string ("reference-id",
                                    &reference_id),
           GNUNET_JSON_spec_mark_optional (
-            GNUNET_JSON_spec_string ("expired_at",
+            GNUNET_JSON_spec_string ("expired-at",
                                      &expired_at),
             NULL),
           GNUNET_JSON_spec_end ()
@@ -1012,25 +1195,11 @@ handle_proof_finished (void *cls,
                                NULL, NULL))
         {
           GNUNET_break_op (0);
-          json_dumpf (j,
-                      stderr,
-                      JSON_INDENT (2));
-          proof_reply_error (ph,
-                             inquiry_id,
-                             MHD_HTTP_BAD_GATEWAY,
-                             "persona-invalid-response",
-                             GNUNET_JSON_PACK (
-                               GNUNET_JSON_pack_uint64 ("persona_http_status",
-                                                        response_code),
-                               GNUNET_JSON_pack_string ("persona_inquiry_id",
-                                                        inquiry_id),
-                               TALER_JSON_pack_ec (
-                                 TALER_EC_EXCHANGE_KYC_GENERIC_PROVIDER_UNEXPECTED_REPLY),
-                               GNUNET_JSON_pack_string ("detail",
-                                                        "data-attributes"),
-                               GNUNET_JSON_pack_allow_null (
-                                 GNUNET_JSON_pack_object_incref ("data",
-                                                                 (json_t *) data))));
+          return_invalid_response (ph,
+                                   response_code,
+                                   inquiry_id,
+                                   "data-attributes",
+                                   data);
           GNUNET_JSON_parse_free (ispec);
           GNUNET_JSON_parse_free (spec);
           break;
@@ -1046,23 +1215,11 @@ handle_proof_finished (void *cls,
                (idr != ph->process_row) )
           {
             GNUNET_break_op (0);
-            proof_reply_error (ph,
-                               inquiry_id,
-                               MHD_HTTP_BAD_GATEWAY,
-                               "persona-invalid-response",
-                               GNUNET_JSON_PACK (
-                                 GNUNET_JSON_pack_uint64 ("persona_http_status",
-                                                          response_code),
-                                 GNUNET_JSON_pack_string ("persona_inquiry_id",
-                                                          inquiry_id),
-                                 TALER_JSON_pack_ec (
-                                   TALER_EC_EXCHANGE_KYC_GENERIC_PROVIDER_UNEXPECTED_REPLY),
-                                 GNUNET_JSON_pack_string ("detail",
-                                                          "data-attributes-reference_id"),
-                                 GNUNET_JSON_pack_allow_null (
-                                   GNUNET_JSON_pack_object_incref ("data",
-                                                                   (json_t *)
-                                                                   data))));
+            return_invalid_response (ph,
+                                     response_code,
+                                     inquiry_id,
+                                     "data-attributes-reference_id",
+                                     data);
             GNUNET_JSON_parse_free (ispec);
             GNUNET_JSON_parse_free (spec);
             break;
@@ -1073,23 +1230,11 @@ handle_proof_finished (void *cls,
                          ph->inquiry_id))
         {
           GNUNET_break_op (0);
-          proof_reply_error (ph,
-                             inquiry_id,
-                             MHD_HTTP_BAD_GATEWAY,
-                             "persona-invalid-response",
-                             GNUNET_JSON_PACK (
-                               GNUNET_JSON_pack_uint64 ("persona_http_status",
-                                                        response_code),
-                               GNUNET_JSON_pack_string ("persona_inquiry_id",
-                                                        inquiry_id),
-                               TALER_JSON_pack_ec (
-                                 TALER_EC_EXCHANGE_KYC_GENERIC_PROVIDER_UNEXPECTED_REPLY),
-                               GNUNET_JSON_pack_string ("detail",
-                                                        "data-id"),
-                               GNUNET_JSON_pack_allow_null (
-                                 GNUNET_JSON_pack_object_incref ("data",
-                                                                 (json_t *)
-                                                                 data))));
+          return_invalid_response (ph,
+                                   response_code,
+                                   inquiry_id,
+                                   "data-id",
+                                   data);
           GNUNET_JSON_parse_free (ispec);
           GNUNET_JSON_parse_free (spec);
           break;
@@ -1099,9 +1244,7 @@ handle_proof_finished (void *cls,
           json_object_get (
             json_object_get (
               json_object_get (
-                json_object_get (
-                  data,
-                  "relationships"),
+                relationships,
                 "account"),
               "data"),
             "id"));
@@ -1109,21 +1252,22 @@ handle_proof_finished (void *cls,
         if (0 != strcmp (status,
                          "completed"))
         {
-          proof_generic_reply (ph,
-                               TALER_KYCLOGIC_STATUS_FAILED,
-                               account_id,
-                               inquiry_id,
-                               MHD_HTTP_OK,
-                               "persona-kyc-failed",
-                               GNUNET_JSON_PACK (
-                                 GNUNET_JSON_pack_uint64 ("persona_http_status",
-                                                          response_code),
-                                 GNUNET_JSON_pack_string ("persona_inquiry_id",
-                                                          inquiry_id),
-                                 GNUNET_JSON_pack_allow_null (
-                                   GNUNET_JSON_pack_object_incref ("data",
-                                                                   (json_t *)
-                                                                   data))));
+          proof_generic_reply (
+            ph,
+            TALER_KYCLOGIC_STATUS_FAILED,
+            account_id,
+            inquiry_id,
+            MHD_HTTP_OK,
+            "persona-kyc-failed",
+            GNUNET_JSON_PACK (
+              GNUNET_JSON_pack_uint64 ("persona_http_status",
+                                       response_code),
+              GNUNET_JSON_pack_string ("persona_inquiry_id",
+                                       inquiry_id),
+              GNUNET_JSON_pack_allow_null (
+                GNUNET_JSON_pack_object_incref ("data",
+                                                (json_t *)
+                                                data))));
           GNUNET_JSON_parse_free (ispec);
           GNUNET_JSON_parse_free (spec);
           break;
@@ -1132,33 +1276,30 @@ handle_proof_finished (void *cls,
         if (NULL == account_id)
         {
           GNUNET_break_op (0);
-          json_dumpf (data,
-                      stderr,
-                      JSON_INDENT (2));
-          proof_reply_error (ph,
-                             inquiry_id,
-                             MHD_HTTP_BAD_GATEWAY,
-                             "persona-invalid-response",
-                             GNUNET_JSON_PACK (
-                               GNUNET_JSON_pack_uint64 ("persona_http_status",
-                                                        response_code),
-                               GNUNET_JSON_pack_string ("persona_inquiry_id",
-                                                        inquiry_id),
-                               TALER_JSON_pack_ec (
-                                 TALER_EC_EXCHANGE_KYC_GENERIC_PROVIDER_UNEXPECTED_REPLY),
-                               GNUNET_JSON_pack_string ("detail",
-                                                        "data-relationships-account-data-id"),
-                               GNUNET_JSON_pack_allow_null (
-                                 GNUNET_JSON_pack_object_incref ("data",
-                                                                 (json_t *)
-                                                                 data))));
+          return_invalid_response (ph,
+                                   response_code,
+                                   inquiry_id,
+                                   "data-relationships-account-data-id",
+                                   data);
           break;
         }
 
         {
           struct MHD_Response *resp;
           struct GNUNET_TIME_Absolute expiration;
+          json_t *attr;
 
+          attr = convert_attributes (attributes);
+          if (NULL == attr)
+          {
+            GNUNET_break_op (0);
+            return_invalid_response (ph,
+                                     response_code,
+                                     inquiry_id,
+                                     "data-relationships-account-data-id",
+                                     data);
+            break;
+          }
           expiration = GNUNET_TIME_relative_to_absolute (ph->pd->validity);
           resp = MHD_create_response_from_buffer (0,
                                                   "",
@@ -1173,8 +1314,10 @@ handle_proof_finished (void *cls,
                   account_id,
                   inquiry_id,
                   expiration,
+                  attr,
                   MHD_HTTP_SEE_OTHER,
                   resp);
+          json_decref (attr);
         }
         GNUNET_JSON_parse_free (ispec);
       }
@@ -1192,59 +1335,61 @@ handle_proof_finished (void *cls,
     json_dumpf (j,
                 stderr,
                 JSON_INDENT (2));
-    proof_reply_error (ph,
-                       ph->inquiry_id,
-                       MHD_HTTP_BAD_GATEWAY,
-                       "persona-logic-failure",
-                       GNUNET_JSON_PACK (
-                         GNUNET_JSON_pack_uint64 ("persona_http_status",
-                                                  response_code),
-                         TALER_JSON_pack_ec (
-                           TALER_EC_EXCHANGE_KYC_GENERIC_PROVIDER_UNEXPECTED_REPLY),
+    proof_reply_error (
+      ph,
+      ph->inquiry_id,
+      MHD_HTTP_BAD_GATEWAY,
+      "persona-logic-failure",
+      GNUNET_JSON_PACK (
+        GNUNET_JSON_pack_uint64 ("persona_http_status",
+                                 response_code),
+        TALER_JSON_pack_ec (
+          TALER_EC_EXCHANGE_KYC_GENERIC_PROVIDER_UNEXPECTED_REPLY),
 
-                         GNUNET_JSON_pack_allow_null (
-                           GNUNET_JSON_pack_object_incref ("data",
-                                                           (json_t *)
-                                                           data))));
+        GNUNET_JSON_pack_allow_null (
+          GNUNET_JSON_pack_object_incref ("data",
+                                          (json_t *)
+                                          data))));
     break;
   case MHD_HTTP_UNAUTHORIZED:
     /* These are failures of the exchange operator */
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Refused access with HTTP status code %u\n",
                 (unsigned int) response_code);
-    proof_reply_error (ph,
-                       ph->inquiry_id,
-                       MHD_HTTP_INTERNAL_SERVER_ERROR,
-                       "persona-exchange-unauthorized",
-                       GNUNET_JSON_PACK (
-                         GNUNET_JSON_pack_uint64 ("persona_http_status",
-                                                  response_code),
-                         TALER_JSON_pack_ec (
-                           TALER_EC_EXCHANGE_KYC_GENERIC_PROVIDER_ACCESS_REFUSED),
-                         GNUNET_JSON_pack_allow_null (
-                           GNUNET_JSON_pack_object_incref ("data",
-                                                           (json_t *)
-                                                           data))));
+    proof_reply_error (
+      ph,
+      ph->inquiry_id,
+      MHD_HTTP_INTERNAL_SERVER_ERROR,
+      "persona-exchange-unauthorized",
+      GNUNET_JSON_PACK (
+        GNUNET_JSON_pack_uint64 ("persona_http_status",
+                                 response_code),
+        TALER_JSON_pack_ec (
+          TALER_EC_EXCHANGE_KYC_GENERIC_PROVIDER_ACCESS_REFUSED),
+        GNUNET_JSON_pack_allow_null (
+          GNUNET_JSON_pack_object_incref ("data",
+                                          (json_t *)
+                                          data))));
     break;
   case MHD_HTTP_PAYMENT_REQUIRED:
     /* These are failures of the exchange operator */
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Refused access with HTTP status code %u\n",
                 (unsigned int) response_code);
-
-    proof_reply_error (ph,
-                       ph->inquiry_id,
-                       MHD_HTTP_INTERNAL_SERVER_ERROR,
-                       "persona-exchange-unpaid",
-                       GNUNET_JSON_PACK (
-                         GNUNET_JSON_pack_uint64 ("persona_http_status",
-                                                  response_code),
-                         TALER_JSON_pack_ec (
-                           TALER_EC_EXCHANGE_KYC_GENERIC_PROVIDER_ACCESS_REFUSED),
-                         GNUNET_JSON_pack_allow_null (
-                           GNUNET_JSON_pack_object_incref ("data",
-                                                           (json_t *)
-                                                           data))));
+    proof_reply_error (
+      ph,
+      ph->inquiry_id,
+      MHD_HTTP_INTERNAL_SERVER_ERROR,
+      "persona-exchange-unpaid",
+      GNUNET_JSON_PACK (
+        GNUNET_JSON_pack_uint64 ("persona_http_status",
+                                 response_code),
+        TALER_JSON_pack_ec (
+          TALER_EC_EXCHANGE_KYC_GENERIC_PROVIDER_ACCESS_REFUSED),
+        GNUNET_JSON_pack_allow_null (
+          GNUNET_JSON_pack_object_incref ("data",
+                                          (json_t *)
+                                          data))));
     break;
   case MHD_HTTP_REQUEST_TIMEOUT:
     /* These are networking issues */
@@ -1254,19 +1399,20 @@ handle_proof_finished (void *cls,
     json_dumpf (j,
                 stderr,
                 JSON_INDENT (2));
-    proof_reply_error (ph,
-                       ph->inquiry_id,
-                       MHD_HTTP_GATEWAY_TIMEOUT,
-                       "persona-network-timeout",
-                       GNUNET_JSON_PACK (
-                         GNUNET_JSON_pack_uint64 ("persona_http_status",
-                                                  response_code),
-                         TALER_JSON_pack_ec (
-                           TALER_EC_EXCHANGE_KYC_GENERIC_PROVIDER_TIMEOUT),
-                         GNUNET_JSON_pack_allow_null (
-                           GNUNET_JSON_pack_object_incref ("data",
-                                                           (json_t *)
-                                                           data))));
+    proof_reply_error (
+      ph,
+      ph->inquiry_id,
+      MHD_HTTP_GATEWAY_TIMEOUT,
+      "persona-network-timeout",
+      GNUNET_JSON_PACK (
+        GNUNET_JSON_pack_uint64 ("persona_http_status",
+                                 response_code),
+        TALER_JSON_pack_ec (
+          TALER_EC_EXCHANGE_KYC_GENERIC_PROVIDER_TIMEOUT),
+        GNUNET_JSON_pack_allow_null (
+          GNUNET_JSON_pack_object_incref ("data",
+                                          (json_t *)
+                                          data))));
     break;
   case MHD_HTTP_TOO_MANY_REQUESTS:
     /* This is a load issue */
@@ -1276,19 +1422,20 @@ handle_proof_finished (void *cls,
     json_dumpf (j,
                 stderr,
                 JSON_INDENT (2));
-    proof_reply_error (ph,
-                       ph->inquiry_id,
-                       MHD_HTTP_SERVICE_UNAVAILABLE,
-                       "persona-load-failure",
-                       GNUNET_JSON_PACK (
-                         GNUNET_JSON_pack_uint64 ("persona_http_status",
-                                                  response_code),
-                         TALER_JSON_pack_ec (
-                           TALER_EC_EXCHANGE_KYC_GENERIC_PROVIDER_RATE_LIMIT_EXCEEDED),
-                         GNUNET_JSON_pack_allow_null (
-                           GNUNET_JSON_pack_object_incref ("data",
-                                                           (json_t *)
-                                                           data))));
+    proof_reply_error (
+      ph,
+      ph->inquiry_id,
+      MHD_HTTP_SERVICE_UNAVAILABLE,
+      "persona-load-failure",
+      GNUNET_JSON_PACK (
+        GNUNET_JSON_pack_uint64 ("persona_http_status",
+                                 response_code),
+        TALER_JSON_pack_ec (
+          TALER_EC_EXCHANGE_KYC_GENERIC_PROVIDER_RATE_LIMIT_EXCEEDED),
+        GNUNET_JSON_pack_allow_null (
+          GNUNET_JSON_pack_object_incref ("data",
+                                          (json_t *)
+                                          data))));
     break;
   case MHD_HTTP_INTERNAL_SERVER_ERROR:
     /* This is an issue with Persona */
@@ -1298,19 +1445,20 @@ handle_proof_finished (void *cls,
     json_dumpf (j,
                 stderr,
                 JSON_INDENT (2));
-    proof_reply_error (ph,
-                       ph->inquiry_id,
-                       MHD_HTTP_BAD_GATEWAY,
-                       "persona-provider-failure",
-                       GNUNET_JSON_PACK (
-                         GNUNET_JSON_pack_uint64 ("persona_http_status",
-                                                  response_code),
-                         TALER_JSON_pack_ec (
-                           TALER_EC_EXCHANGE_KYC_PROOF_BACKEND_ERROR),
-                         GNUNET_JSON_pack_allow_null (
-                           GNUNET_JSON_pack_object_incref ("data",
-                                                           (json_t *)
-                                                           data))));
+    proof_reply_error (
+      ph,
+      ph->inquiry_id,
+      MHD_HTTP_BAD_GATEWAY,
+      "persona-provider-failure",
+      GNUNET_JSON_PACK (
+        GNUNET_JSON_pack_uint64 ("persona_http_status",
+                                 response_code),
+        TALER_JSON_pack_ec (
+          TALER_EC_EXCHANGE_KYC_PROOF_BACKEND_ERROR),
+        GNUNET_JSON_pack_allow_null (
+          GNUNET_JSON_pack_object_incref ("data",
+                                          (json_t *)
+                                          data))));
     break;
   default:
     /* This is an issue with Persona */
@@ -1320,21 +1468,22 @@ handle_proof_finished (void *cls,
     json_dumpf (j,
                 stderr,
                 JSON_INDENT (2));
-    proof_reply_error (ph,
-                       ph->inquiry_id,
-                       MHD_HTTP_BAD_GATEWAY,
-                       "persona-invalid-response",
-                       GNUNET_JSON_PACK (
-                         GNUNET_JSON_pack_uint64 ("persona_http_status",
-                                                  response_code),
-                         TALER_JSON_pack_ec (
-                           TALER_EC_EXCHANGE_KYC_GENERIC_PROVIDER_UNEXPECTED_REPLY),
-                         GNUNET_JSON_pack_string ("detail",
-                                                  "data-relationships-account-data-id"),
-                         GNUNET_JSON_pack_allow_null (
-                           GNUNET_JSON_pack_object_incref ("data",
-                                                           (json_t *)
-                                                           data))));
+    proof_reply_error (
+      ph,
+      ph->inquiry_id,
+      MHD_HTTP_BAD_GATEWAY,
+      "persona-invalid-response",
+      GNUNET_JSON_PACK (
+        GNUNET_JSON_pack_uint64 ("persona_http_status",
+                                 response_code),
+        TALER_JSON_pack_ec (
+          TALER_EC_EXCHANGE_KYC_GENERIC_PROVIDER_UNEXPECTED_REPLY),
+        GNUNET_JSON_pack_string ("detail",
+                                 "data-relationships-account-data-id"),
+        GNUNET_JSON_pack_allow_null (
+          GNUNET_JSON_pack_object_incref ("data",
+                                          (json_t *)
+                                          data))));
     break;
   }
   persona_proof_cancel (ph);
@@ -1444,6 +1593,7 @@ persona_webhook_cancel (struct TALER_KYCLOGIC_WebhookHandle *wh)
  * @param status status to return
  * @param account_id account to return
  * @param inquiry_id inquiry ID to supply
+ * @param attr KYC attribute data for the client
  * @param http_status HTTP status to use
  */
 static void
@@ -1451,6 +1601,7 @@ webhook_generic_reply (struct TALER_KYCLOGIC_WebhookHandle *wh,
                        enum TALER_KYCLOGIC_KycStatus status,
                        const char *account_id,
                        const char *inquiry_id,
+                       const json_t *attr,
                        unsigned int http_status)
 {
   struct MHD_Response *resp;
@@ -1467,11 +1618,12 @@ webhook_generic_reply (struct TALER_KYCLOGIC_WebhookHandle *wh,
   wh->cb (wh->cb_cls,
           wh->process_row,
           &wh->h_payto,
-          account_id,
           wh->pd->section,
+          account_id,
           inquiry_id,
           status,
           expiration,
+          attr,
           http_status,
           resp);
 }
@@ -1493,13 +1645,14 @@ webhook_reply_error (struct TALER_KYCLOGIC_WebhookHandle *wh,
                          TALER_KYCLOGIC_STATUS_PROVIDER_FAILED,
                          NULL, /* user id */
                          inquiry_id,
+                         NULL, /* attributes */
                          http_status);
 }
 
 
 /**
  * Function called when we're done processing the
- * HTTP "/verifications/{verification_id}" request.
+ * HTTP "/api/v1/inquiries/{inquiry_id}" request.
  *
  * @param cls the `struct TALER_KYCLOGIC_WebhookHandle`
  * @param response_code HTTP response code, 0 on error
@@ -1524,6 +1677,7 @@ handle_webhook_finished (void *cls,
       const char *account_id;
       const char *type = NULL;
       json_t *attributes;
+      json_t *relationships;
       struct GNUNET_JSON_Specification spec[] = {
         GNUNET_JSON_spec_string ("type",
                                  &type),
@@ -1531,6 +1685,8 @@ handle_webhook_finished (void *cls,
                                  &inquiry_id),
         GNUNET_JSON_spec_json ("attributes",
                                &attributes),
+        GNUNET_JSON_spec_json ("relationships",
+                               &relationships),
         GNUNET_JSON_spec_end ()
       };
 
@@ -1559,14 +1715,15 @@ handle_webhook_finished (void *cls,
         struct GNUNET_JSON_Specification ispec[] = {
           GNUNET_JSON_spec_string ("status",
                                    &status),
-          GNUNET_JSON_spec_string ("reference_id",
+          GNUNET_JSON_spec_string ("reference-id",
                                    &reference_id),
           GNUNET_JSON_spec_mark_optional (
-            GNUNET_JSON_spec_string ("expired_at",
+            GNUNET_JSON_spec_string ("expired-at",
                                      &expired_at),
             NULL),
           GNUNET_JSON_spec_end ()
         };
+        json_t *attr;
 
         if (GNUNET_OK !=
             GNUNET_JSON_parse (attributes,
@@ -1620,9 +1777,7 @@ handle_webhook_finished (void *cls,
           json_object_get (
             json_object_get (
               json_object_get (
-                json_object_get (
-                  data,
-                  "relationships"),
+                relationships,
                 "account"),
               "data"),
             "id"));
@@ -1634,6 +1789,7 @@ handle_webhook_finished (void *cls,
                                  TALER_KYCLOGIC_STATUS_FAILED,
                                  account_id,
                                  inquiry_id,
+                                 NULL,
                                  MHD_HTTP_OK);
           GNUNET_JSON_parse_free (ispec);
           GNUNET_JSON_parse_free (spec);
@@ -1652,11 +1808,14 @@ handle_webhook_finished (void *cls,
           break;
         }
 
+        attr = convert_attributes (attributes);
         webhook_generic_reply (wh,
                                TALER_KYCLOGIC_STATUS_SUCCESS,
                                account_id,
                                inquiry_id,
+                               attr,
                                MHD_HTTP_OK);
+        json_decref (attr);
         GNUNET_JSON_parse_free (ispec);
       }
       GNUNET_JSON_parse_free (spec);
@@ -1771,6 +1930,7 @@ async_webhook_reply (void *cls)
           wh->inquiry_id, /* provider legi ID */
           TALER_KYCLOGIC_STATUS_PROVIDER_FAILED,
           GNUNET_TIME_UNIT_ZERO_ABS, /* expiration */
+          NULL,
           wh->response_code,
           wh->resp);
   persona_webhook_cancel (wh);
@@ -1852,13 +2012,13 @@ persona_webhook (void *cls,
   wh->ps = ps;
   wh->connection = connection;
   wh->pd = pd;
-
   auth_header = MHD_lookup_connection_value (connection,
                                              MHD_HEADER_KIND,
                                              MHD_HTTP_HEADER_AUTHORIZATION);
   if ( (NULL != ps->webhook_token) &&
-       (0 != strcmp (ps->webhook_token,
-                     auth_header)) )
+       ( (NULL == auth_header) ||
+         (0 != strcmp (ps->webhook_token,
+                       auth_header)) ) )
   {
     GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
                 "Invalid authorization header `%s' received for Persona webhook\n",
@@ -1890,7 +2050,7 @@ persona_webhook (void *cls,
                     "payload"),
                   "data"),
                 "relationships"),
-              "inquiry_template"),
+              "inquiry-template"),
             "data"),
           "id"));
   if (NULL == wh->template_id)
@@ -1933,7 +2093,6 @@ persona_webhook (void *cls,
     return wh;
   }
 
-
   persona_inquiry_id
     = json_string_value (
         json_object_get (
