@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2020 Taler Systems SA
+  Copyright (C) 2014-2022 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -75,12 +75,6 @@ enum AuditorHandleState
 };
 
 
-/**
- * Data for the request to get the /version of a auditor.
- */
-struct VersionRequest;
-
-
 /**
  * Handle to the auditor
  */
@@ -111,7 +105,12 @@ struct TALER_AUDITOR_Handle
    * Data for the request to get the /version of a auditor,
    * NULL once we are past stage #MHS_INIT.
    */
-  struct VersionRequest *vr;
+  struct GNUNET_CURL_Job *vr;
+
+  /**
+   * The url for the @e vr job.
+   */
+  char *vr_url;
 
   /**
    * Task for retrying /version request.
@@ -144,50 +143,12 @@ struct TALER_AUDITOR_Handle
 
 /* ***************** Internal /version fetching ************* */
 
-/**
- * Data for the request to get the /version of a auditor.
- */
-struct VersionRequest
-{
-  /**
-   * The connection to auditor this request handle will use
-   */
-  struct TALER_AUDITOR_Handle *auditor;
-
-  /**
-   * The url for this handle
-   */
-  char *url;
-
-  /**
-   * Entry for this request with the `struct GNUNET_CURL_Context`.
-   */
-  struct GNUNET_CURL_Job *job;
-
-};
-
-
-/**
- * Release memory occupied by a version request.
- * Note that this does not cancel the request
- * itself.
- *
- * @param vr request to free
- */
-static void
-free_version_request (struct VersionRequest *vr)
-{
-  GNUNET_free (vr->url);
-  GNUNET_free (vr);
-}
-
-
 /**
  * Decode the JSON in @a resp_obj from the /version response and store the data
  * in the @a key_data.
  *
  * @param[in] resp_obj JSON object to parse
- * @param[out] auditor where to store the results we decoded
+ * @param[in,out] auditor where to store the results we decoded
  * @param[out] vc where to store version compatibility data
  * @return #TALER_EC_NONE on success
  */
@@ -200,6 +161,7 @@ decode_version_json (const json_t *resp_obj,
   unsigned int age;
   unsigned int revision;
   unsigned int current;
+  char dummy;
   const char *ver;
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_string ("version",
@@ -224,14 +186,16 @@ decode_version_json (const json_t *resp_obj,
     return TALER_EC_GENERIC_JSON_INVALID;
   }
   if (3 != sscanf (ver,
-                   "%u:%u:%u",
+                   "%u:%u:%u%c",
                    &current,
                    &revision,
-                   &age))
+                   &age,
+                   &dummy))
   {
     GNUNET_break_op (0);
     return TALER_EC_GENERIC_VERSION_MALFORMED;
   }
+  GNUNET_free (auditor->version);
   auditor->version = GNUNET_strdup (ver);
   vi->version = auditor->version;
   *vc = TALER_AUDITOR_VC_MATCH;
@@ -264,7 +228,7 @@ request_version (void *cls);
  * Callback used when downloading the reply to a /version request
  * is complete.
  *
- * @param cls the `struct VersionRequest`
+ * @param cls the `struct TALER_AUDITOR_Handle`
  * @param response_code HTTP response code, 0 on error
  * @param gresp_obj parsed JSON result, NULL on error, must be a `const json_t *`
  */
@@ -273,18 +237,18 @@ version_completed_cb (void *cls,
                       long response_code,
                       const void *gresp_obj)
 {
+  struct TALER_AUDITOR_Handle *auditor = cls;
   const json_t *resp_obj = gresp_obj;
-  struct VersionRequest *vr = cls;
-  struct TALER_AUDITOR_Handle *auditor = vr->auditor;
   enum TALER_AUDITOR_VersionCompatibility vc;
   struct TALER_AUDITOR_HttpResponse hr = {
     .reply = resp_obj,
     .http_status = (unsigned int) response_code
   };
 
+  auditor->vr = NULL;
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Received version from URL `%s' with status %ld.\n",
-              vr->url,
+              auditor->url,
               response_code);
   vc = TALER_AUDITOR_VC_PROTOCOL_ERROR;
   switch (response_code)
@@ -293,8 +257,6 @@ version_completed_cb (void *cls,
   case MHD_HTTP_INTERNAL_SERVER_ERROR:
     /* NOTE: this design is debatable. We MAY want to throw this error at the
        client. We may then still additionally internally re-try. */
-    free_version_request (vr);
-    auditor->vr = NULL;
     GNUNET_assert (NULL == auditor->retry_task);
     auditor->retry_delay = EXCHANGE_LIB_BACKOFF (auditor->retry_delay);
     auditor->retry_task = GNUNET_SCHEDULER_add_delayed (auditor->retry_delay,
@@ -333,11 +295,9 @@ version_completed_cb (void *cls,
   if (MHD_HTTP_OK != response_code)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "/version failed for auditor %p: %u!\n",
-                auditor,
+                "/version failed for auditor %s: %u!\n",
+                auditor->url,
                 (unsigned int) response_code);
-    auditor->vr = NULL;
-    free_version_request (vr);
     auditor->state = MHS_FAILED;
     /* notify application that we failed */
     auditor->version_cb (auditor->version_cb_cls,
@@ -346,14 +306,11 @@ version_completed_cb (void *cls,
                          vc);
     return;
   }
-
-  auditor->vr = NULL;
-  free_version_request (vr);
   TALER_LOG_DEBUG ("Switching auditor state to 'version'\n");
   auditor->state = MHS_VERSION;
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-              "Auditor %p is now READY!\n",
-              auditor);
+              "Auditor %s is now READY!\n",
+              auditor->url);
   /* notify application about the key information */
   auditor->version_cb (auditor->version_cb_cls,
                        &hr,
@@ -362,107 +319,6 @@ version_completed_cb (void *cls,
 }
 
 
-/* ********************* library internal API ********* */
-
-
-/**
- * Get the context of a auditor.
- *
- * @param h the auditor handle to query
- * @return ctx context to execute jobs in
- */
-struct GNUNET_CURL_Context *
-TALER_AUDITOR_handle_to_context_ (struct TALER_AUDITOR_Handle *h)
-{
-  return h->ctx;
-}
-
-
-/**
- * Check if the handle is ready to process requests.
- *
- * @param h the auditor handle to query
- * @return #GNUNET_YES if we are ready, #GNUNET_NO if not
- */
-int
-TALER_AUDITOR_handle_is_ready_ (struct TALER_AUDITOR_Handle *h)
-{
-  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-              "Checking if auditor %p (%s) is now ready: %s\n",
-              h,
-              h->url,
-              (MHD_VERSION == h->state) ? "yes" : "no");
-  return (MHS_VERSION == h->state) ? GNUNET_YES : GNUNET_NO;
-}
-
-
-/**
- * Obtain the URL to use for an API request.
- *
- * @param h handle for the auditor
- * @param path Taler API path (i.e. "/deposit-confirmation")
- * @return the full URL to use with cURL
- */
-char *
-TALER_AUDITOR_path_to_url_ (struct TALER_AUDITOR_Handle *h,
-                            const char *path)
-{
-  char *ret;
-  GNUNET_assert ('/' == path[0]);
-  ret = TALER_url_join (h->url,
-                        path + 1,
-                        NULL);
-  GNUNET_assert (NULL != ret);
-  return ret;
-}
-
-
-/* ********************* public API ******************* */
-
-
-/**
- * Initialise a connection to the auditor. Will connect to the
- * auditor and obtain information about the auditor's master public
- * key and the auditor's auditor.  The respective information will
- * be passed to the @a version_cb once available, and all future
- * interactions with the auditor will be checked to be signed
- * (where appropriate) by the respective master key.
- *
- * @param ctx the context
- * @param url HTTP base URL for the auditor
- * @param version_cb function to call with the
- *        auditor's version information
- * @param version_cb_cls closure for @a version_cb
- * @return the auditor handle; NULL upon error
- */
-struct TALER_AUDITOR_Handle *
-TALER_AUDITOR_connect (struct GNUNET_CURL_Context *ctx,
-                       const char *url,
-                       TALER_AUDITOR_VersionCallback version_cb,
-                       void *version_cb_cls)
-{
-  struct TALER_AUDITOR_Handle *auditor;
-
-  /* Disable 100 continue processing */
-  GNUNET_break (GNUNET_OK ==
-                GNUNET_CURL_append_header (ctx,
-                                           "Expect:"));
-  auditor = GNUNET_new (struct TALER_AUDITOR_Handle);
-  auditor->retry_delay = GNUNET_TIME_UNIT_SECONDS; /* start slowly */
-  auditor->ctx = ctx;
-  auditor->url = GNUNET_strdup (url);
-  auditor->version_cb = version_cb;
-  auditor->version_cb_cls = version_cb_cls;
-  auditor->retry_task = GNUNET_SCHEDULER_add_now (&request_version,
-                                                  auditor);
-  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-              "Connecting to auditor at URL `%s' (%p).\n",
-              url,
-              auditor);
-  return auditor;
-}
-
-
 /**
  * Initiate download of /version from the auditor.
  *
@@ -472,19 +328,14 @@ static void
 request_version (void *cls)
 {
   struct TALER_AUDITOR_Handle *auditor = cls;
-  struct VersionRequest *vr;
   CURL *eh;
 
   auditor->retry_task = NULL;
   GNUNET_assert (NULL == auditor->vr);
-  vr = GNUNET_new (struct VersionRequest);
-  vr->auditor = auditor;
-  vr->url = TALER_AUDITOR_path_to_url_ (auditor,
-                                        "/version");
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Requesting auditor version with URL `%s'.\n",
-              vr->url);
-  eh = TALER_AUDITOR_curl_easy_get_ (vr->url);
+              auditor->vr_url);
+  eh = TALER_AUDITOR_curl_easy_get_ (auditor->vr_url);
   if (NULL == eh)
   {
     GNUNET_break (0);
@@ -498,38 +349,98 @@ request_version (void *cls)
                 curl_easy_setopt (eh,
                                   CURLOPT_TIMEOUT,
                                   (long) 300));
-  vr->job = GNUNET_CURL_job_add (auditor->ctx,
-                                 eh,
-                                 &version_completed_cb,
-                                 vr);
-  auditor->vr = vr;
+  auditor->vr = GNUNET_CURL_job_add (auditor->ctx,
+                                     eh,
+                                     &version_completed_cb,
+                                     auditor);
+}
+
+
+/* ********************* library internal API ********* */
+
+
+struct GNUNET_CURL_Context *
+TALER_AUDITOR_handle_to_context_ (struct TALER_AUDITOR_Handle *h)
+{
+  return h->ctx;
+}
+
+
+enum GNUNET_GenericReturnValue
+TALER_AUDITOR_handle_is_ready_ (struct TALER_AUDITOR_Handle *h)
+{
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Checking if auditor at `%s` is now ready: %s\n",
+              h->url,
+              (MHD_VERSION == h->state) ? "yes" : "no");
+  return (MHS_VERSION == h->state) ? GNUNET_YES : GNUNET_NO;
+}
+
+
+char *
+TALER_AUDITOR_path_to_url_ (struct TALER_AUDITOR_Handle *h,
+                            const char *path)
+{
+  GNUNET_assert ('/' == path[0]);
+  return TALER_url_join (h->url,
+                         path + 1,
+                         NULL);
+}
+
+
+/* ********************* public API ******************* */
+
+
+struct TALER_AUDITOR_Handle *
+TALER_AUDITOR_connect (struct GNUNET_CURL_Context *ctx,
+                       const char *url,
+                       TALER_AUDITOR_VersionCallback version_cb,
+                       void *version_cb_cls)
+{
+  struct TALER_AUDITOR_Handle *auditor;
+
+  auditor = GNUNET_new (struct TALER_AUDITOR_Handle);
+  auditor->version_cb = version_cb;
+  auditor->version_cb_cls = version_cb_cls;
+  auditor->retry_delay = GNUNET_TIME_UNIT_SECONDS; /* start slowly */
+  auditor->ctx = ctx;
+  auditor->url = GNUNET_strdup (url);
+  auditor->vr_url = TALER_AUDITOR_path_to_url_ (auditor,
+                                                "/version");
+  if (NULL == auditor->vr_url)
+  {
+    GNUNET_break (0);
+    GNUNET_free (auditor->url);
+    GNUNET_free (auditor);
+    return NULL;
+  }
+  auditor->retry_task = GNUNET_SCHEDULER_add_now (&request_version,
+                                                  auditor);
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Connecting to auditor at URL `%s'.\n",
+              url);
+  return auditor;
 }
 
 
-/**
- * Disconnect from the auditor
- *
- * @param auditor the auditor handle
- */
 void
 TALER_AUDITOR_disconnect (struct TALER_AUDITOR_Handle *auditor)
 {
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-              "Disconnecting from auditor at URL `%s' (%p).\n",
-              auditor->url,
-              auditor);
+              "Disconnecting from auditor at URL `%s'.\n",
+              auditor->url);
   if (NULL != auditor->vr)
   {
-    GNUNET_CURL_job_cancel (auditor->vr->job);
-    free_version_request (auditor->vr);
+    GNUNET_CURL_job_cancel (auditor->vr);
     auditor->vr = NULL;
   }
-  GNUNET_free (auditor->version);
   if (NULL != auditor->retry_task)
   {
     GNUNET_SCHEDULER_cancel (auditor->retry_task);
     auditor->retry_task = NULL;
   }
+  GNUNET_free (auditor->version);
+  GNUNET_free (auditor->vr_url);
   GNUNET_free (auditor->url);
   GNUNET_free (auditor);
 }
