@@ -25,7 +25,7 @@
 #include <gnunet/gnunet_curl_lib.h>
 #include "taler_auditor_service.h"
 #include "taler_exchange_service.h"
-#include "taler_crypto_lib.h"
+#include "taler_util.h"
 #include "taler_curl_lib.h"
 
 /**
