@@ -109,6 +109,8 @@ handle_purse_get_finished (void *cls,
           &no_deposit),
         TALER_JSON_spec_amount_any ("balance",
                                     &dr.details.success.balance),
+        GNUNET_JSON_spec_timestamp ("purse_expiration",
+                                    &dr.details.success.purse_expiration),
         GNUNET_JSON_spec_fixed_auto ("exchange_pub",
                                      &exchange_pub),
         GNUNET_JSON_spec_fixed_auto ("exchange_sig",
