@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2015-2020 Taler Systems SA
+  Copyright (C) 2015-2022 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -60,6 +60,11 @@ struct TALER_EXCHANGE_RefreshesRevealHandle
    */
   struct GNUNET_CURL_Job *job;
 
+  /**
+   * Exchange-contributed values to the operation.
+   */
+  struct TALER_ExchangeWithdrawValues *alg_values;
+
   /**
    * Function to call with the result.
    */
@@ -73,7 +78,7 @@ struct TALER_EXCHANGE_RefreshesRevealHandle
   /**
    * Actual information about the melt operation.
    */
-  struct MeltData *md;
+  struct MeltData md;
 
   /**
    * The index selected by the exchange in cut-and-choose to not be revealed.
@@ -84,28 +89,28 @@ struct TALER_EXCHANGE_RefreshesRevealHandle
 
 
 /**
- * We got a 200 OK response for the /refreshes/$RCH/reveal operation.
- * Extract the coin signatures and return them to the caller.
- * The signatures we get from the exchange is for the blinded value.
- * Thus, we first must unblind them and then should verify their
- * validity.
+ * We got a 200 OK response for the /refreshes/$RCH/reveal operation.  Extract
+ * the coin signatures and return them to the caller.  The signatures we get
+ * from the exchange is for the blinded value.  Thus, we first must unblind
+ * them and then should verify their validity.
  *
  * If everything checks out, we return the unblinded signatures
  * to the application via the callback.
  *
  * @param rrh operation handle
  * @param json reply from the exchange
- * @param[out] sigs array of length `num_fresh_coins`, initialized to contain RSA signatures
+ * @param[out] rcis array of length `num_fresh_coins`, initialized to contain the coin data
  * @return #GNUNET_OK on success, #GNUNET_SYSERR on errors
  */
-static int
+static enum GNUNET_GenericReturnValue
 refresh_reveal_ok (struct TALER_EXCHANGE_RefreshesRevealHandle *rrh,
                    const json_t *json,
-                   struct TALER_DenominationSignature *sigs)
+                   struct TALER_EXCHANGE_RevealedCoinInfo *rcis)
 {
   json_t *jsona;
   struct GNUNET_JSON_Specification outer_spec[] = {
-    GNUNET_JSON_spec_json ("ev_sigs", &jsona),
+    GNUNET_JSON_spec_json ("ev_sigs",
+                           &jsona),
     GNUNET_JSON_spec_end ()
   };
 
@@ -124,31 +129,50 @@ refresh_reveal_ok (struct TALER_EXCHANGE_RefreshesRevealHandle *rrh,
     GNUNET_JSON_parse_free (outer_spec);
     return GNUNET_SYSERR;
   }
-  if (rrh->md->num_fresh_coins != json_array_size (jsona))
+  if (rrh->md.num_fresh_coins != json_array_size (jsona))
   {
     /* Number of coins generated does not match our expectation */
     GNUNET_break_op (0);
     GNUNET_JSON_parse_free (outer_spec);
     return GNUNET_SYSERR;
   }
-  for (unsigned int i = 0; i<rrh->md->num_fresh_coins; i++)
+  for (unsigned int i = 0; i<rrh->md.num_fresh_coins; i++)
   {
-    const struct TALER_PlanchetSecretsP *fc;
-    struct TALER_DenominationPublicKey *pk;
+    struct TALER_EXCHANGE_RevealedCoinInfo *rci =
+      &rcis[i];
+    const struct FreshCoinData *fcd = &rrh->md.fcds[i];
+    const struct TALER_DenominationPublicKey *pk;
     json_t *jsonai;
-    struct GNUNET_CRYPTO_RsaSignature *blind_sig;
+    struct TALER_BlindedDenominationSignature blind_sig;
     struct TALER_CoinSpendPublicKeyP coin_pub;
-    struct GNUNET_HashCode coin_hash;
+    struct TALER_CoinPubHashP coin_hash;
     struct GNUNET_JSON_Specification spec[] = {
-      GNUNET_JSON_spec_rsa_signature ("ev_sig", &blind_sig),
+      TALER_JSON_spec_blinded_denom_sig ("ev_sig",
+                                         &blind_sig),
       GNUNET_JSON_spec_end ()
     };
     struct TALER_FreshCoin coin;
+    union TALER_DenominationBlindingKeyP bks;
 
-    fc = &rrh->md->fresh_coins[rrh->noreveal_index][i];
-    pk = &rrh->md->fresh_pks[i];
+    rci->ps = fcd->ps[rrh->noreveal_index];
+    rci->bks = fcd->bks[rrh->noreveal_index];
+    rci->age_commitment_proof = fcd->age_commitment_proof[rrh->noreveal_index];
+    rci->h_age_commitment = NULL;
+    pk = &fcd->fresh_pk;
     jsonai = json_array_get (jsona, i);
+
     GNUNET_assert (NULL != jsonai);
+    GNUNET_assert (
+      (NULL != rrh->md.melted_coin.age_commitment_proof) ==
+      (NULL != rci->age_commitment_proof));
+
+    if (NULL != rci->age_commitment_proof)
+    {
+      rci->h_age_commitment = GNUNET_new (struct TALER_AgeCommitmentHash);
+      TALER_age_commitment_hash (
+        &rci->age_commitment_proof->commitment,
+        rci->h_age_commitment);
+    }
 
     if (GNUNET_OK !=
         GNUNET_JSON_parse (jsonai,
@@ -160,27 +184,36 @@ refresh_reveal_ok (struct TALER_EXCHANGE_RefreshesRevealHandle *rrh,
       return GNUNET_SYSERR;
     }
 
+    TALER_planchet_setup_coin_priv (&rci->ps,
+                                    &rrh->alg_values[i],
+                                    &rci->coin_priv);
+    TALER_planchet_blinding_secret_create (&rci->ps,
+                                           &rrh->alg_values[i],
+                                           &bks);
     /* needed to verify the signature, and we didn't store it earlier,
        hence recomputing it here... */
-    GNUNET_CRYPTO_eddsa_key_get_public (&fc->coin_priv.eddsa_priv,
+    GNUNET_CRYPTO_eddsa_key_get_public (&rci->coin_priv.eddsa_priv,
                                         &coin_pub.eddsa_pub);
-    GNUNET_CRYPTO_hash (&coin_pub.eddsa_pub,
-                        sizeof (struct GNUNET_CRYPTO_EcdsaPublicKey),
-                        &coin_hash);
+    TALER_coin_pub_hash (&coin_pub,
+                         rci->h_age_commitment,
+                         &coin_hash);
     if (GNUNET_OK !=
         TALER_planchet_to_coin (pk,
-                                blind_sig,
-                                fc,
+                                &blind_sig,
+                                &bks,
+                                &rci->coin_priv,
+                                rci->h_age_commitment,
                                 &coin_hash,
+                                &rrh->alg_values[i],
                                 &coin))
     {
       GNUNET_break_op (0);
-      GNUNET_CRYPTO_rsa_signature_free (blind_sig);
+      GNUNET_JSON_parse_free (spec);
       GNUNET_JSON_parse_free (outer_spec);
       return GNUNET_SYSERR;
     }
-    GNUNET_CRYPTO_rsa_signature_free (blind_sig);
-    sigs[i] = coin.sig;
+    GNUNET_JSON_parse_free (spec);
+    rci->sig = coin.sig;
   }
   GNUNET_JSON_parse_free (outer_spec);
   return GNUNET_OK;
@@ -202,113 +235,97 @@ handle_refresh_reveal_finished (void *cls,
 {
   struct TALER_EXCHANGE_RefreshesRevealHandle *rrh = cls;
   const json_t *j = response;
-  struct TALER_EXCHANGE_HttpResponse hr = {
-    .reply = j,
-    .http_status = (unsigned int) response_code
+  struct TALER_EXCHANGE_RevealResult rr = {
+    .hr.reply = j,
+    .hr.http_status = (unsigned int) response_code
   };
 
   rrh->job = NULL;
   switch (response_code)
   {
   case 0:
-    hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
+    rr.hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
     break;
   case MHD_HTTP_OK:
     {
-      struct TALER_DenominationSignature sigs[rrh->md->num_fresh_coins];
-      int ret;
+      struct TALER_EXCHANGE_RevealedCoinInfo rcis[rrh->md.num_fresh_coins];
+      enum GNUNET_GenericReturnValue ret;
 
-      memset (sigs, 0, sizeof (sigs));
+      memset (rcis,
+              0,
+              sizeof (rcis));
       ret = refresh_reveal_ok (rrh,
                                j,
-                               sigs);
+                               rcis);
       if (GNUNET_OK != ret)
       {
-        hr.http_status = 0;
-        hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
+        rr.hr.http_status = 0;
+        rr.hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
+        break;
       }
       else
       {
+        GNUNET_assert (rrh->noreveal_index < TALER_CNC_KAPPA);
+        rr.details.success.num_coins = rrh->md.num_fresh_coins;
+        rr.details.success.coins = rcis;
         rrh->reveal_cb (rrh->reveal_cb_cls,
-                        &hr,
-                        rrh->md->num_fresh_coins,
-                        rrh->md->fresh_coins[rrh->noreveal_index],
-                        sigs);
+                        &rr);
         rrh->reveal_cb = NULL;
       }
-      for (unsigned int i = 0; i<rrh->md->num_fresh_coins; i++)
-        if (NULL != sigs[i].rsa_signature)
-          GNUNET_CRYPTO_rsa_signature_free (sigs[i].rsa_signature);
+      for (unsigned int i = 0; i<rrh->md.num_fresh_coins; i++)
+        TALER_denom_sig_free (&rcis[i].sig);
       TALER_EXCHANGE_refreshes_reveal_cancel (rrh);
       return;
     }
   case MHD_HTTP_BAD_REQUEST:
     /* This should never happen, either us or the exchange is buggy
        (or API version conflict); just pass JSON reply to the application */
-    hr.ec = TALER_JSON_get_error_code (j);
-    hr.hint = TALER_JSON_get_error_hint (j);
+    rr.hr.ec = TALER_JSON_get_error_code (j);
+    rr.hr.hint = TALER_JSON_get_error_hint (j);
     break;
   case MHD_HTTP_CONFLICT:
     /* Nothing really to verify, exchange says our reveal is inconsistent
        with our commitment, so either side is buggy; we
        should pass the JSON reply to the application */
-    hr.ec = TALER_JSON_get_error_code (j);
-    hr.hint = TALER_JSON_get_error_hint (j);
+    rr.hr.ec = TALER_JSON_get_error_code (j);
+    rr.hr.hint = TALER_JSON_get_error_hint (j);
+    break;
+  case MHD_HTTP_GONE:
+    /* Server claims key expired or has been revoked */
+    rr.hr.ec = TALER_JSON_get_error_code (j);
+    rr.hr.hint = TALER_JSON_get_error_hint (j);
     break;
   case MHD_HTTP_INTERNAL_SERVER_ERROR:
     /* Server had an internal issue; we should retry, but this API
        leaves this to the application */
-    hr.ec = TALER_JSON_get_error_code (j);
-    hr.hint = TALER_JSON_get_error_hint (j);
+    rr.hr.ec = TALER_JSON_get_error_code (j);
+    rr.hr.hint = TALER_JSON_get_error_hint (j);
     break;
   default:
     /* unexpected response code */
     GNUNET_break_op (0);
-    hr.ec = TALER_JSON_get_error_code (j);
-    hr.hint = TALER_JSON_get_error_hint (j);
+    rr.hr.ec = TALER_JSON_get_error_code (j);
+    rr.hr.hint = TALER_JSON_get_error_hint (j);
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Unexpected response code %u/%d\n",
+                "Unexpected response code %u/%d for exchange refreshes reveal\n",
                 (unsigned int) response_code,
-                (int) hr.ec);
+                (int) rr.hr.ec);
     break;
   }
   if (NULL != rrh->reveal_cb)
     rrh->reveal_cb (rrh->reveal_cb_cls,
-                    &hr,
-                    0,
-                    NULL,
-                    NULL);
+                    &rr);
   TALER_EXCHANGE_refreshes_reveal_cancel (rrh);
 }
 
 
-/**
- * Submit a /refresh/reval request to the exchange and get the exchange's
- * response.
- *
- * This API is typically used by a wallet.  Note that to ensure that
- * no money is lost in case of hardware failures, the provided
- * arguments should have been committed to persistent storage
- * prior to calling this function.
- *
- * @param exchange the exchange handle; the exchange must be ready to operate
- * @param refresh_data_length size of the @a refresh_data (returned
- *        in the `res_size` argument from #TALER_EXCHANGE_refresh_prepare())
- * @param refresh_data the refresh data as returned from
-          #TALER_EXCHANGE_refresh_prepare())
- * @param noreveal_index response from the exchange to the
- *        #TALER_EXCHANGE_melt() invocation
- * @param reveal_cb the callback to call with the final result of the
- *        refresh operation
- * @param reveal_cb_cls closure for the above callback
- * @return a handle for this request; NULL if the argument was invalid.
- *         In this case, neither callback will be called.
- */
 struct TALER_EXCHANGE_RefreshesRevealHandle *
 TALER_EXCHANGE_refreshes_reveal (
   struct TALER_EXCHANGE_Handle *exchange,
-  size_t refresh_data_length,
-  const char *refresh_data,
+  const struct TALER_RefreshMasterSecretP *rms,
+  const struct TALER_EXCHANGE_RefreshData *rd,
+  unsigned int num_coins,
+  const struct TALER_ExchangeWithdrawValues *alg_values,
   uint32_t noreveal_index,
   TALER_EXCHANGE_RefreshesRevealCallback reveal_cb,
   void *reveal_cb_cls)
@@ -319,12 +336,14 @@ TALER_EXCHANGE_refreshes_reveal (
   json_t *coin_evs;
   json_t *reveal_obj;
   json_t *link_sigs;
+  json_t *old_age_commitment = NULL;
   CURL *eh;
   struct GNUNET_CURL_Context *ctx;
-  struct MeltData *md;
-  struct TALER_TransferPublicKeyP transfer_pub;
+  struct MeltData md;
   char arg_str[sizeof (struct TALER_RefreshCommitmentP) * 2 + 32];
+  bool send_rms = false;
 
+  GNUNET_assert (num_coins == rd->fresh_pks_len);
   if (noreveal_index >= TALER_CNC_KAPPA)
   {
     /* We check this here, as it would be really bad to below just
@@ -340,77 +359,58 @@ TALER_EXCHANGE_refreshes_reveal (
     GNUNET_break (0);
     return NULL;
   }
-  md = TALER_EXCHANGE_deserialize_melt_data_ (refresh_data,
-                                              refresh_data_length);
-  if (NULL == md)
+  if (GNUNET_OK !=
+      TALER_EXCHANGE_get_melt_data_ (rms,
+                                     rd,
+                                     alg_values,
+                                     &md))
   {
     GNUNET_break (0);
     return NULL;
   }
 
-  /* now transfer_pub */
-  GNUNET_CRYPTO_ecdhe_key_get_public (
-    &md->melted_coin.transfer_priv[noreveal_index].ecdhe_priv,
-    &transfer_pub.ecdhe_pub);
-
   /* now new_denoms */
   GNUNET_assert (NULL != (new_denoms_h = json_array ()));
   GNUNET_assert (NULL != (coin_evs = json_array ()));
   GNUNET_assert (NULL != (link_sigs = json_array ()));
-  for (unsigned int i = 0; i<md->num_fresh_coins; i++)
+  for (unsigned int i = 0; i<md.num_fresh_coins; i++)
   {
-    struct GNUNET_HashCode denom_hash;
-    struct TALER_PlanchetDetail pd;
-    struct GNUNET_HashCode c_hash;
+    const struct TALER_RefreshCoinData *rcd = &md.rcd[noreveal_index][i];
+    struct TALER_DenominationHashP denom_hash;
 
-    GNUNET_CRYPTO_rsa_public_key_hash (md->fresh_pks[i].rsa_public_key,
-                                       &denom_hash);
+    if (TALER_DENOMINATION_CS == md.fcds[i].fresh_pk.cipher)
+      send_rms = true;
+    TALER_denom_pub_hash (&md.fcds[i].fresh_pk,
+                          &denom_hash);
     GNUNET_assert (0 ==
                    json_array_append_new (new_denoms_h,
                                           GNUNET_JSON_from_data_auto (
                                             &denom_hash)));
-
-    if (GNUNET_OK !=
-        TALER_planchet_prepare (&md->fresh_pks[i],
-                                &md->fresh_coins[noreveal_index][i],
-                                &c_hash,
-                                &pd))
-    {
-      /* This should have been noticed during the preparation stage. */
-      GNUNET_break (0);
-      json_decref (new_denoms_h);
-      json_decref (coin_evs);
-      return NULL;
-    }
     GNUNET_assert (0 ==
-                   json_array_append_new (coin_evs,
-                                          GNUNET_JSON_from_data (pd.coin_ev,
-                                                                 pd.coin_ev_size)));
-
-    /* compute link signature */
+                   json_array_append_new (
+                     coin_evs,
+                     GNUNET_JSON_PACK (
+                       TALER_JSON_pack_blinded_planchet (
+                         NULL,
+                         &rcd->blinded_planchet))));
     {
       struct TALER_CoinSpendSignatureP link_sig;
-      struct TALER_LinkDataPS ldp;
+      struct TALER_BlindedCoinHashP bch;
 
-      ldp.purpose.size = htonl (sizeof (ldp));
-      ldp.purpose.purpose = htonl (TALER_SIGNATURE_WALLET_COIN_LINK);
-      ldp.h_denom_pub = denom_hash;
-      GNUNET_CRYPTO_eddsa_key_get_public (&md->melted_coin.coin_priv.eddsa_priv,
-                                          &ldp.old_coin_pub.eddsa_pub);
-      ldp.transfer_pub = transfer_pub;
-      GNUNET_CRYPTO_hash (pd.coin_ev,
-                          pd.coin_ev_size,
-                          &ldp.coin_envelope_hash);
-      GNUNET_CRYPTO_eddsa_sign (&md->melted_coin.coin_priv.eddsa_priv,
-                                &ldp,
-                                &link_sig.eddsa_signature);
+      TALER_coin_ev_hash (&rcd->blinded_planchet,
+                          &denom_hash,
+                          &bch);
+      TALER_wallet_link_sign (
+        &denom_hash,
+        &md.transfer_pub[noreveal_index],
+        &bch,
+        &md.melted_coin.coin_priv,
+        &link_sig);
       GNUNET_assert (0 ==
-                     json_array_append_new (link_sigs,
-                                            GNUNET_JSON_from_data_auto (
-                                              &link_sig)));
+                     json_array_append_new (
+                       link_sigs,
+                       GNUNET_JSON_from_data_auto (&link_sig)));
     }
-
-    GNUNET_free (pd.coin_ev);
   }
 
   /* build array of transfer private keys */
@@ -419,41 +419,60 @@ TALER_EXCHANGE_refreshes_reveal (
   {
     if (j == noreveal_index)
     {
-      /* This is crucial: exclude the transfer key for the
-   noreval index! */
+      /* This is crucial: exclude the transfer key for the noreval index! */
       continue;
     }
     GNUNET_assert (0 ==
                    json_array_append_new (transfer_privs,
                                           GNUNET_JSON_from_data_auto (
-                                            &md->melted_coin.transfer_priv[j])));
+                                            &md.transfer_priv[j])));
+  }
+
+  /* build array of old age commitment, if applicable */
+  GNUNET_assert ((NULL == rd->melt_age_commitment_proof) ==
+                 (NULL == rd->melt_h_age_commitment));
+  if (NULL != rd->melt_age_commitment_proof)
+  {
+    GNUNET_assert (NULL != (old_age_commitment = json_array ()));
+
+    for (size_t i = 0; i < rd->melt_age_commitment_proof->commitment.num; i++)
+    {
+      GNUNET_assert (0 ==
+                     json_array_append_new (
+                       old_age_commitment,
+                       GNUNET_JSON_from_data_auto (
+                         &rd->melt_age_commitment_proof->
+                         commitment.keys[i])));
+    }
   }
 
   /* build main JSON request */
-  reveal_obj = json_pack ("{s:o, s:o, s:o, s:o, s:o}",
-                          "transfer_pub",
-                          GNUNET_JSON_from_data_auto (&transfer_pub),
-                          "transfer_privs",
-                          transfer_privs,
-                          "link_sigs",
-                          link_sigs,
-                          "new_denoms_h",
-                          new_denoms_h,
-                          "coin_evs",
-                          coin_evs);
-  if (NULL == reveal_obj)
-  {
-    GNUNET_break (0);
-    return NULL;
-  }
-
+  reveal_obj = GNUNET_JSON_PACK (
+    GNUNET_JSON_pack_data_auto ("transfer_pub",
+                                &md.transfer_pub[noreveal_index]),
+    GNUNET_JSON_pack_allow_null (
+      send_rms
+      ? GNUNET_JSON_pack_data_auto ("rms",
+                                    rms)
+      : GNUNET_JSON_pack_string ("rms",
+                                 NULL)),
+    GNUNET_JSON_pack_allow_null (
+      GNUNET_JSON_pack_array_steal ("old_age_commitment",
+                                    old_age_commitment)),
+    GNUNET_JSON_pack_array_steal ("transfer_privs",
+                                  transfer_privs),
+    GNUNET_JSON_pack_array_steal ("link_sigs",
+                                  link_sigs),
+    GNUNET_JSON_pack_array_steal ("new_denoms_h",
+                                  new_denoms_h),
+    GNUNET_JSON_pack_array_steal ("coin_evs",
+                                  coin_evs));
   {
     char pub_str[sizeof (struct TALER_RefreshCommitmentP) * 2];
     char *end;
 
-    end = GNUNET_STRINGS_data_to_string (&md->rc,
-                                         sizeof (struct
-                                                 TALER_RefreshCommitmentP),
+    end = GNUNET_STRINGS_data_to_string (&md.rc,
+                                         sizeof (md.rc),
                                          pub_str,
                                          sizeof (pub_str));
     *end = '\0';
@@ -469,8 +488,20 @@ TALER_EXCHANGE_refreshes_reveal (
   rrh->reveal_cb = reveal_cb;
   rrh->reveal_cb_cls = reveal_cb_cls;
   rrh->md = md;
+  rrh->alg_values = GNUNET_memdup (alg_values,
+                                   md.num_fresh_coins
+                                   * sizeof (struct
+                                             TALER_ExchangeWithdrawValues));
   rrh->url = TEAH_path_to_url (rrh->exchange,
                                arg_str);
+  if (NULL == rrh->url)
+  {
+    json_decref (reveal_obj);
+    TALER_EXCHANGE_free_melt_data_ (&md);
+    GNUNET_free (rrh->alg_values);
+    GNUNET_free (rrh);
+    return NULL;
+  }
 
   eh = TALER_EXCHANGE_curl_easy_get_ (rrh->url);
   if ( (NULL == eh) ||
@@ -483,6 +514,8 @@ TALER_EXCHANGE_refreshes_reveal (
     if (NULL != eh)
       curl_easy_cleanup (eh);
     json_decref (reveal_obj);
+    TALER_EXCHANGE_free_melt_data_ (&md);
+    GNUNET_free (rrh->alg_values);
     GNUNET_free (rrh->url);
     GNUNET_free (rrh);
     return NULL;
@@ -498,12 +531,6 @@ TALER_EXCHANGE_refreshes_reveal (
 }
 
 
-/**
- * Cancel a refresh reveal request.  This function cannot be used
- * on a request handle if the callback was already invoked.
- *
- * @param rrh the refresh reval handle
- */
 void
 TALER_EXCHANGE_refreshes_reveal_cancel (
   struct TALER_EXCHANGE_RefreshesRevealHandle *rrh)
@@ -513,10 +540,10 @@ TALER_EXCHANGE_refreshes_reveal_cancel (
     GNUNET_CURL_job_cancel (rrh->job);
     rrh->job = NULL;
   }
+  GNUNET_free (rrh->alg_values);
   GNUNET_free (rrh->url);
   TALER_curl_easy_post_finished (&rrh->ctx);
-  TALER_EXCHANGE_free_melt_data_ (rrh->md); /* does not free 'md' itself */
-  GNUNET_free (rrh->md);
+  TALER_EXCHANGE_free_melt_data_ (&rrh->md);
   GNUNET_free (rrh);
 }
 
