@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2020 Taler Systems SA
+  Copyright (C) 2014-2022 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -129,7 +129,6 @@ parse_fees (json_t *fees)
 
     if (0 == (len = json_array_size (fee_array)))
     {
-      GNUNET_break_op (0);
       GNUNET_free (fe);
       continue; /* skip */
     }
@@ -144,14 +143,14 @@ parse_fees (json_t *fees)
       struct GNUNET_JSON_Specification spec[] = {
         GNUNET_JSON_spec_fixed_auto ("sig",
                                      &wa->master_sig),
-        TALER_JSON_spec_amount ("wire_fee",
-                                &wa->wire_fee),
-        TALER_JSON_spec_amount ("closing_fee",
-                                &wa->closing_fee),
-        TALER_JSON_spec_absolute_time ("start_date",
-                                       &wa->start_date),
-        TALER_JSON_spec_absolute_time ("end_date",
-                                       &wa->end_date),
+        TALER_JSON_spec_amount_any ("wire_fee",
+                                    &wa->fees.wire),
+        TALER_JSON_spec_amount_any ("closing_fee",
+                                    &wa->fees.closing),
+        GNUNET_JSON_spec_timestamp ("start_date",
+                                    &wa->start_date),
+        GNUNET_JSON_spec_timestamp ("end_date",
+                                    &wa->end_date),
         GNUNET_JSON_spec_end ()
       };
 
@@ -220,6 +219,7 @@ handle_wire_finished (void *cls,
   {
   case 0:
     hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
+    wh->exchange->wire_error_count++;
     break;
   case MHD_HTTP_OK:
     {
@@ -229,11 +229,15 @@ handle_wire_finished (void *cls,
       struct FeeMap *fm;
       const struct TALER_EXCHANGE_Keys *key_state;
       struct GNUNET_JSON_Specification spec[] = {
-        GNUNET_JSON_spec_json ("accounts", &accounts),
-        GNUNET_JSON_spec_json ("fees", &fees),
+        GNUNET_JSON_spec_json ("accounts",
+                               &accounts),
+        GNUNET_JSON_spec_json ("fees",
+                               &fees),
         GNUNET_JSON_spec_end ()
       };
 
+      wh->exchange->wire_error_count = 0;
+
       if (GNUNET_OK !=
           GNUNET_JSON_parse (j,
                              spec,
@@ -274,8 +278,10 @@ handle_wire_finished (void *cls,
           struct TALER_EXCHANGE_WireAccount *wa = &was[i];
           json_t *account;
           struct GNUNET_JSON_Specification spec_account[] = {
-            GNUNET_JSON_spec_string ("payto_uri", &wa->payto_uri),
-            GNUNET_JSON_spec_fixed_auto ("master_sig", &wa->master_sig),
+            GNUNET_JSON_spec_string ("payto_uri",
+                                     &wa->payto_uri),
+            GNUNET_JSON_spec_fixed_auto ("master_sig",
+                                         &wa->master_sig),
             GNUNET_JSON_spec_end ()
           };
           char *method;
@@ -357,11 +363,13 @@ handle_wire_finished (void *cls,
     break;
   default:
     /* unexpected response code */
+    if (MHD_HTTP_GATEWAY_TIMEOUT == response_code)
+      wh->exchange->wire_error_count++;
     GNUNET_break_op (0);
     hr.ec = TALER_JSON_get_error_code (j);
     hr.hint = TALER_JSON_get_error_hint (j);
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Unexpected response code %u/%d\n",
+                "Unexpected response code %u/%d for exchange wire\n",
                 (unsigned int) response_code,
                 (int) hr.ec);
     break;
@@ -375,6 +383,20 @@ handle_wire_finished (void *cls,
 }
 
 
+/**
+ * Compute the network timeout for the next request to /wire.
+ *
+ * @param exchange the exchange handle
+ * @returns the timeout in seconds (for use by CURL)
+ */
+static long
+get_wire_timeout_seconds (struct TALER_EXCHANGE_Handle *exchange)
+{
+  return GNUNET_MIN (60,
+                     5 + (1L << exchange->wire_error_count));
+}
+
+
 /**
  * Obtain information about a exchange's wire instructions.
  * A exchange may provide wire instructions for creating
@@ -416,6 +438,11 @@ TALER_EXCHANGE_wire (struct TALER_EXCHANGE_Handle *exchange,
   wh->cb_cls = wire_cb_cls;
   wh->url = TEAH_path_to_url (exchange,
                               "/wire");
+  if (NULL == wh->url)
+  {
+    GNUNET_free (wh);
+    return NULL;
+  }
   eh = TALER_EXCHANGE_curl_easy_get_ (wh->url);
   if (NULL == eh)
   {
@@ -424,6 +451,10 @@ TALER_EXCHANGE_wire (struct TALER_EXCHANGE_Handle *exchange,
     GNUNET_free (wh);
     return NULL;
   }
+  GNUNET_break (CURLE_OK ==
+                curl_easy_setopt (eh,
+                                  CURLOPT_TIMEOUT,
+                                  get_wire_timeout_seconds (wh->exchange)));
   ctx = TEAH_handle_to_context (exchange);
   wh->job = GNUNET_CURL_job_add_with_ct_json (ctx,
                                               eh,
