@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2020 Taler Systems SA
+  Copyright (C) 2014-2022 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -95,14 +95,14 @@ struct TALER_EXCHANGE_Withdraw2Handle
  * @param json reply from the exchange
  * @return #GNUNET_OK on success, #GNUNET_SYSERR on errors
  */
-static int
+static enum GNUNET_GenericReturnValue
 reserve_withdraw_ok (struct TALER_EXCHANGE_Withdraw2Handle *wh,
                      const json_t *json)
 {
-  struct GNUNET_CRYPTO_RsaSignature *blind_sig;
+  struct TALER_BlindedDenominationSignature blind_sig;
   struct GNUNET_JSON_Specification spec[] = {
-    GNUNET_JSON_spec_rsa_signature ("ev_sig",
-                                    &blind_sig),
+    TALER_JSON_spec_blinded_denom_sig ("ev_sig",
+                                       &blind_sig),
     GNUNET_JSON_spec_end ()
   };
   struct TALER_EXCHANGE_HttpResponse hr = {
@@ -122,7 +122,7 @@ reserve_withdraw_ok (struct TALER_EXCHANGE_Withdraw2Handle *wh,
   /* signature is valid, return it to the application */
   wh->cb (wh->cb_cls,
           &hr,
-          blind_sig);
+          &blind_sig);
   /* make sure callback isn't called again after return */
   wh->cb = NULL;
   GNUNET_JSON_parse_free (spec);
@@ -142,17 +142,19 @@ reserve_withdraw_ok (struct TALER_EXCHANGE_Withdraw2Handle *wh,
  * @param json reply from the exchange
  * @return #GNUNET_OK on success, #GNUNET_SYSERR on errors
  */
-static int
+static enum GNUNET_GenericReturnValue
 reserve_withdraw_payment_required (
   struct TALER_EXCHANGE_Withdraw2Handle *wh,
   const json_t *json)
 {
   struct TALER_Amount balance;
-  struct TALER_Amount balance_from_history;
+  struct TALER_Amount total_in_from_history;
+  struct TALER_Amount total_out_from_history;
   json_t *history;
   size_t len;
   struct GNUNET_JSON_Specification spec[] = {
-    TALER_JSON_spec_amount ("balance", &balance),
+    TALER_JSON_spec_amount_any ("balance",
+                                &balance),
     GNUNET_JSON_spec_end ()
   };
 
@@ -176,15 +178,15 @@ reserve_withdraw_payment_required (
      total incoming and outgoing amounts */
   len = json_array_size (history);
   {
-    struct TALER_EXCHANGE_ReserveHistory *rhistory;
+    struct TALER_EXCHANGE_ReserveHistoryEntry *rhistory;
 
     /* Use heap allocation as "len" may be very big and thus this may
        not fit on the stack. Use "GNUNET_malloc_large" as a malicious
        exchange may theoretically try to crash us by giving a history
        that does not fit into our memory. */
-    rhistory = GNUNET_malloc_large (sizeof (struct
-                                            TALER_EXCHANGE_ReserveHistory)
-                                    * len);
+    rhistory = GNUNET_malloc_large (
+      sizeof (struct TALER_EXCHANGE_ReserveHistoryEntry)
+      * len);
     if (NULL == rhistory)
     {
       GNUNET_break (0);
@@ -196,7 +198,8 @@ reserve_withdraw_payment_required (
                                               history,
                                               &wh->reserve_pub,
                                               balance.currency,
-                                              &balance_from_history,
+                                              &total_in_from_history,
+                                              &total_out_from_history,
                                               len,
                                               rhistory))
     {
@@ -209,14 +212,6 @@ reserve_withdraw_payment_required (
                                          len);
   }
 
-  if (0 !=
-      TALER_amount_cmp (&balance_from_history,
-                        &balance))
-  {
-    /* exchange cannot add up balances!? */
-    GNUNET_break_op (0);
-    return GNUNET_SYSERR;
-  }
   /* Check that funds were really insufficient */
   if (0 >= TALER_amount_cmp (&wh->requested_amount,
                              &balance))
@@ -275,6 +270,22 @@ handle_reserve_withdraw_finished (void *cls,
     hr.ec = TALER_JSON_get_error_code (j);
     hr.hint = TALER_JSON_get_error_hint (j);
     break;
+  case MHD_HTTP_FORBIDDEN:
+    GNUNET_break_op (0);
+    /* Nothing really to verify, exchange says one of the signatures is
+       invalid; as we checked them, this should never happen, we
+       should pass the JSON reply to the application */
+    hr.ec = TALER_JSON_get_error_code (j);
+    hr.hint = TALER_JSON_get_error_hint (j);
+    break;
+  case MHD_HTTP_NOT_FOUND:
+    /* Nothing really to verify, the exchange basically just says
+       that it doesn't know this reserve.  Can happen if we
+       query before the wire transfer went through.
+       We should simply pass the JSON reply to the application. */
+    hr.ec = TALER_JSON_get_error_code (j);
+    hr.hint = TALER_JSON_get_error_hint (j);
+    break;
   case MHD_HTTP_CONFLICT:
     /* The exchange says that the reserve has insufficient funds;
        check the signatures in the history... */
@@ -292,21 +303,35 @@ handle_reserve_withdraw_finished (void *cls,
       hr.hint = TALER_JSON_get_error_hint (j);
     }
     break;
-  case MHD_HTTP_FORBIDDEN:
-    GNUNET_break_op (0);
-    /* Nothing really to verify, exchange says one of the signatures is
-       invalid; as we checked them, this should never happen, we
-       should pass the JSON reply to the application */
+  case MHD_HTTP_GONE:
+    /* could happen if denomination was revoked */
+    /* Note: one might want to check /keys for revocation
+       signature here, alas tricky in case our /keys
+       is outdated => left to clients */
     hr.ec = TALER_JSON_get_error_code (j);
     hr.hint = TALER_JSON_get_error_hint (j);
     break;
-  case MHD_HTTP_NOT_FOUND:
-    /* Nothing really to verify, the exchange basically just says
-       that it doesn't know this reserve.  Can happen if we
-       query before the wire transfer went through.
-       We should simply pass the JSON reply to the application. */
-    hr.ec = TALER_JSON_get_error_code (j);
-    hr.hint = TALER_JSON_get_error_hint (j);
+  case MHD_HTTP_UNAVAILABLE_FOR_LEGAL_REASONS:
+    /* only validate reply is well-formed */
+    {
+      uint64_t ptu;
+      struct GNUNET_JSON_Specification spec[] = {
+        GNUNET_JSON_spec_uint64 ("requirement_row",
+                                 &ptu),
+        GNUNET_JSON_spec_end ()
+      };
+
+      if (GNUNET_OK !=
+          GNUNET_JSON_parse (j,
+                             spec,
+                             NULL, NULL))
+      {
+        GNUNET_break_op (0);
+        hr.http_status = 0;
+        hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
+        break;
+      }
+    }
     break;
   case MHD_HTTP_INTERNAL_SERVER_ERROR:
     /* Server had an internal issue; we should retry, but this API
@@ -320,7 +345,7 @@ handle_reserve_withdraw_finished (void *cls,
     hr.ec = TALER_JSON_get_error_code (j);
     hr.hint = TALER_JSON_get_error_hint (j);
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Unexpected response code %u/%d\n",
+                "Unexpected response code %u/%d for exchange withdraw\n",
                 (unsigned int) response_code,
                 (int) hr.ec);
     break;
@@ -336,25 +361,6 @@ handle_reserve_withdraw_finished (void *cls,
 }
 
 
-/**
- * Withdraw a coin from the exchange using a /reserve/withdraw
- * request.  This API is typically used by a wallet to withdraw a tip
- * where the reserve's signature was created by the merchant already.
- *
- * Note that to ensure that no money is lost in case of hardware
- * failures, the caller must have committed (most of) the arguments to
- * disk before calling, and be ready to repeat the request with the
- * same arguments in case of failures.
- *
- * @param exchange the exchange handle; the exchange must be ready to operate
- * @param pd planchet details of the planchet to withdraw
- * @param reserve_priv private key of the reserve to withdraw from
- * @param res_cb the callback to call when the final result for this request is available
- * @param res_cb_cls closure for @a res_cb
- * @return NULL
- *         if the inputs are invalid (i.e. denomination key not with this exchange).
- *         In this case, the callback is not called.
- */
 struct TALER_EXCHANGE_Withdraw2Handle *
 TALER_EXCHANGE_withdraw2 (
   struct TALER_EXCHANGE_Handle *exchange,
@@ -368,6 +374,7 @@ TALER_EXCHANGE_withdraw2 (
   const struct TALER_EXCHANGE_DenomPublicKey *dk;
   struct TALER_ReserveSignatureP reserve_sig;
   char arg_str[sizeof (struct TALER_ReservePublicKeyP) * 2 + 32];
+  struct TALER_BlindedCoinHashP bch;
 
   keys = TALER_EXCHANGE_get_keys (exchange);
   if (NULL == keys)
@@ -390,7 +397,7 @@ TALER_EXCHANGE_withdraw2 (
   if (0 >
       TALER_amount_add (&wh->requested_amount,
                         &dk->value,
-                        &dk->fee_withdraw))
+                        &dk->fees.withdraw))
   {
     /* Overflow here? Very strange, our CPU must be fried... */
     GNUNET_break (0);
@@ -416,46 +423,41 @@ TALER_EXCHANGE_withdraw2 (
                      "/reserves/%s/withdraw",
                      pub_str);
   }
-  {
-    struct TALER_WithdrawRequestPS req = {
-      .purpose.size = htonl (sizeof (struct TALER_WithdrawRequestPS)),
-      .purpose.purpose = htonl (TALER_SIGNATURE_WALLET_RESERVE_WITHDRAW),
-      .reserve_pub = wh->reserve_pub,
-      .h_denomination_pub = pd->denom_pub_hash
-    };
 
-    TALER_amount_hton (&req.amount_with_fee,
-                       &wh->requested_amount);
-    GNUNET_CRYPTO_hash (pd->coin_ev,
-                        pd->coin_ev_size,
-                        &req.h_coin_envelope);
-    GNUNET_CRYPTO_eddsa_sign (&reserve_priv->eddsa_priv,
-                              &req,
-                              &reserve_sig.eddsa_signature);
+  if (GNUNET_OK !=
+      TALER_coin_ev_hash (&pd->blinded_planchet,
+                          &pd->denom_pub_hash,
+                          &bch))
+  {
+    GNUNET_break (0);
+    GNUNET_free (wh);
+    return NULL;
   }
 
+  TALER_wallet_withdraw_sign (&pd->denom_pub_hash,
+                              &wh->requested_amount,
+                              &bch,
+                              reserve_priv,
+                              &reserve_sig);
   {
-    json_t *withdraw_obj;
-
-    withdraw_obj = json_pack ("{s:o, s:o, s:o}",
-                              "denom_pub_hash",
-                              GNUNET_JSON_from_data_auto (&pd->denom_pub_hash),
-                              "coin_ev",
-                              GNUNET_JSON_from_data (pd->coin_ev,
-                                                     pd->coin_ev_size),
-                              "reserve_sig",
-                              GNUNET_JSON_from_data_auto (&reserve_sig));
-    if (NULL == withdraw_obj)
-    {
-      GNUNET_break (0);
-      GNUNET_free (wh);
-      return NULL;
-    }
+    json_t *withdraw_obj = GNUNET_JSON_PACK (
+      GNUNET_JSON_pack_data_auto ("denom_pub_hash",
+                                  &pd->denom_pub_hash),
+      TALER_JSON_pack_blinded_planchet ("coin_ev",
+                                        &pd->blinded_planchet),
+      GNUNET_JSON_pack_data_auto ("reserve_sig",
+                                  &reserve_sig));
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                 "Attempting to withdraw from reserve %s\n",
                 TALER_B2S (&wh->reserve_pub));
     wh->url = TEAH_path_to_url (exchange,
                                 arg_str);
+    if (NULL == wh->url)
+    {
+      json_decref (withdraw_obj);
+      GNUNET_free (wh);
+      return NULL;
+    }
     {
       CURL *eh;
       struct GNUNET_CURL_Context *ctx;
@@ -488,12 +490,6 @@ TALER_EXCHANGE_withdraw2 (
 }
 
 
-/**
- * Cancel a withdraw status request.  This function cannot be used
- * on a request handle if a response is already served for it.
- *
- * @param wh the withdraw sign request handle
- */
 void
 TALER_EXCHANGE_withdraw2_cancel (struct TALER_EXCHANGE_Withdraw2Handle *wh)
 {
