@@ -29,6 +29,7 @@
 #include "taler_exchange_service.h"
 #include "taler_json_lib.h"
 #include <gnunet/gnunet_util_lib.h>
+#include <gnunet/gnunet_testing_lib.h>
 #include <microhttpd.h>
 #include "taler_bank_service.h"
 #include "taler_fakebank_lib.h"
@@ -40,7 +41,7 @@
  * Configuration file we use.  One (big) configuration is used
  * for the various components for this test.
  */
-#define CONFIG_FILE "test_exchange_api_twisted.conf"
+static char *config_file;
 
 /**
  * (real) Twister URL.  Used at startup time to check if it runs.
@@ -73,7 +74,7 @@ static struct TALER_TESTING_Command
 CMD_EXEC_WIREWATCH (const char *label)
 {
   return TALER_TESTING_cmd_exec_wirewatch (label,
-                                           CONFIG_FILE);
+                                           config_file);
 }
 
 
@@ -123,6 +124,7 @@ run (void *cls,
     TALER_TESTING_cmd_withdraw_amount ("refresh-withdraw-coin",
                                        "refresh-create-reserve",
                                        "EUR:5",
+                                       0, /* age restriction off */
                                        MHD_HTTP_OK),
     TALER_TESTING_cmd_deposit ("refresh-deposit-partial",
                                "refresh-withdraw-coin",
@@ -142,7 +144,7 @@ run (void *cls,
                             NULL),
     /* Trigger 409 Conflict.  */
     TALER_TESTING_cmd_flip_upload ("flip-upload",
-                                   CONFIG_FILE,
+                                   config_file,
                                    "transfer_privs.0"),
     TALER_TESTING_cmd_refresh_reveal ("refresh-(flipped-)reveal",
                                       "refresh-melt",
@@ -155,13 +157,15 @@ run (void *cls,
    * NOTE: not all CMDs actually need the twister,
    * so it may be better to move those into the "main"
    * lib test suite.
-   */struct TALER_TESTING_Command refund[] = {
+   */
+  struct TALER_TESTING_Command refund[] = {
     CMD_TRANSFER_TO_EXCHANGE ("create-reserve-r1",
                               "EUR:5.01"),
     CMD_EXEC_WIREWATCH ("wirewatch-r1"),
     TALER_TESTING_cmd_withdraw_amount ("withdraw-coin-r1",
                                        "create-reserve-r1",
                                        "EUR:5",
+                                       0, /* age restriction off */
                                        MHD_HTTP_OK),
     TALER_TESTING_cmd_deposit ("deposit-refund-1",
                                "withdraw-coin-r1",
@@ -177,13 +181,12 @@ run (void *cls,
                               "USD:5",
                               "deposit-refund-1"),
     TALER_TESTING_cmd_flip_upload ("flip-upload",
-                                   CONFIG_FILE,
+                                   config_file,
                                    "merchant_sig"),
     TALER_TESTING_cmd_refund ("refund-bad-sig",
                               MHD_HTTP_FORBIDDEN,
                               "EUR:5",
                               "deposit-refund-1"),
-
     /* This next deposit CMD is only used to provide a
      * good merchant signature to the next (failing) refund
      * operations.  */
@@ -210,19 +213,19 @@ run (void *cls,
     TALER_TESTING_cmd_end ()
   };
 
+#if 0
   /**
    * Test that we don't get errors when the keys from the exchange
    * are out of date.
    */
   struct TALER_TESTING_Command expired_keys[] = {
     TALER_TESTING_cmd_modify_header_dl ("modify-expiration",
-                                        CONFIG_FILE,
+                                        config_file,
                                         MHD_HTTP_HEADER_EXPIRES,
                                         "Wed, 19 Jan 586524 08:01:49 GMT"),
     TALER_TESTING_cmd_check_keys_pull_all_keys (
       "check-keys-expiration-0",
-      2,
-      5),
+      2),
     /**
      * Run some normal commands after this to make sure everything is fine.
      */
@@ -232,20 +235,33 @@ run (void *cls,
     TALER_TESTING_cmd_withdraw_amount ("withdraw-coin-r2",
                                        "create-reserve-r2",
                                        "EUR:5",
+                                       0, /* age restriction off */
                                        MHD_HTTP_OK),
     TALER_TESTING_cmd_end ()
   };
+#endif
 
   struct TALER_TESTING_Command commands[] = {
+    TALER_TESTING_cmd_wire_add ("add-wire-account",
+                                "payto://x-taler-bank/localhost/2?receiver-name=2",
+                                MHD_HTTP_NO_CONTENT,
+                                false),
+    TALER_TESTING_cmd_exec_offline_sign_keys ("offline-sign-future-keys",
+                                              config_file),
+    TALER_TESTING_cmd_check_keys_pull_all_keys ("refetch /keys",
+                                                1),
     TALER_TESTING_cmd_batch ("refresh-reveal-409-conflict",
                              refresh_409_conflict),
     TALER_TESTING_cmd_batch ("refund",
                              refund),
+#if 0
     TALER_TESTING_cmd_batch ("expired-keys",
                              expired_keys),
+#endif
     TALER_TESTING_cmd_end ()
   };
 
+  (void) cls;
   TALER_TESTING_run_with_fakebank (is,
                                    commands,
                                    bc.exchange_auth.wire_gateway_url);
@@ -271,24 +287,32 @@ int
 main (int argc,
       char *const *argv)
 {
+  char *cipher;
   int ret;
 
+  (void) argc;
   /* These environment variables get in the way... */
   unsetenv ("XDG_DATA_HOME");
   unsetenv ("XDG_CONFIG_HOME");
-  GNUNET_log_setup ("test-exchange-api-twisted",
-                    "DEBUG",
+  GNUNET_log_setup (argv[0],
+                    "INFO",
                     NULL);
+  cipher = GNUNET_TESTING_get_testname_from_underscore (argv[0]);
+  GNUNET_assert (NULL != cipher);
+  GNUNET_asprintf (&config_file,
+                   "test_exchange_api_twisted-%s.conf",
+                   cipher);
+  GNUNET_free (cipher);
   if (GNUNET_OK !=
-      TALER_TESTING_prepare_fakebank (CONFIG_FILE,
+      TALER_TESTING_prepare_fakebank (config_file,
                                       "exchange-account-2",
                                       &bc))
     return 77;
   if (NULL == (twister_url = TALER_TWISTER_prepare_twister
-                               (CONFIG_FILE)))
+                               (config_file)))
     return 77;
-  TALER_TESTING_cleanup_files (CONFIG_FILE);
-  switch (TALER_TESTING_prepare_exchange (CONFIG_FILE,
+  TALER_TESTING_cleanup_files (config_file);
+  switch (TALER_TESTING_prepare_exchange (config_file,
                                           GNUNET_YES,
                                           &ec))
   {
@@ -298,11 +322,11 @@ main (int argc,
   case GNUNET_NO:
     return 77;
   case GNUNET_OK:
-    if (NULL == (twisterd = TALER_TWISTER_run_twister (CONFIG_FILE)))
+    if (NULL == (twisterd = TALER_TWISTER_run_twister (config_file)))
       return 77;
     ret = TALER_TESTING_setup_with_exchange (&run,
                                              NULL,
-                                             CONFIG_FILE);
+                                             config_file);
     purge_process (twisterd);
     GNUNET_free (twister_url);
 
