@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2022 Taler Systems SA
+  Copyright (C) 2014-2023 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as
@@ -23,6 +23,7 @@
  */
 #include "platform.h"
 #include "taler_util.h"
+#include "taler_attributes.h"
 #include "taler_signatures.h"
 #include "taler_exchange_service.h"
 #include "taler_json_lib.h"
@@ -91,15 +92,14 @@ static void
 run (void *cls,
      struct TALER_TESTING_Interpreter *is)
 {
-  /**
-   * Test withdraw.
-   */
   struct TALER_TESTING_Command withdraw[] = {
     CMD_TRANSFER_TO_EXCHANGE ("create-reserve-1",
                               "EUR:15.02"),
     TALER_TESTING_cmd_check_bank_admin_transfer (
       "check-create-reserve-1",
-      "EUR:15.02", bc.user42_payto, bc.exchange_payto,
+      "EUR:15.02",
+      bc.user42_payto,
+      bc.exchange_payto,
       "create-reserve-1"),
     CMD_EXEC_WIREWATCH ("wirewatch-1"),
     TALER_TESTING_cmd_withdraw_amount ("withdraw-coin-1-no-kyc",
@@ -137,6 +137,12 @@ run (void *cls,
                                        "EUR:5",
                                        0, /* age restriction off */
                                        MHD_HTTP_OK),
+    /* Attestations above are bound to the originating *bank* account,
+       not to the reserve (!). Hence, they are NOT found here! */
+    TALER_TESTING_cmd_reserve_get_attestable ("reserve-get-attestable",
+                                              "create-reserve-1",
+                                              MHD_HTTP_NOT_FOUND,
+                                              NULL),
     TALER_TESTING_cmd_end ()
   };
   struct TALER_TESTING_Command spend[] = {
@@ -216,6 +222,16 @@ run (void *cls,
     TALER_TESTING_cmd_check_kyc_get ("wallet-kyc-check",
                                      "wallet-kyc-fail",
                                      MHD_HTTP_NO_CONTENT),
+    TALER_TESTING_cmd_reserve_get_attestable ("wallet-get-attestable",
+                                              "wallet-kyc-fail",
+                                              MHD_HTTP_OK,
+                                              TALER_ATTRIBUTE_FULL_NAME,
+                                              NULL),
+    TALER_TESTING_cmd_reserve_attest ("wallet-get-attest",
+                                      "wallet-kyc-fail",
+                                      MHD_HTTP_OK,
+                                      TALER_ATTRIBUTE_FULL_NAME,
+                                      NULL),
     TALER_TESTING_cmd_end ()
   };
 
@@ -394,7 +410,112 @@ run (void *cls,
       MHD_HTTP_OK),
     TALER_TESTING_cmd_end ()
   };
-
+  struct TALER_TESTING_Command aml[] = {
+    /* Trigger something upon which an AML officer could act */
+    TALER_TESTING_cmd_wallet_kyc_get ("wallet-trigger-kyc-for-aml",
+                                      NULL,
+                                      "EUR:1000",
+                                      MHD_HTTP_UNAVAILABLE_FOR_LEGAL_REASONS),
+    TALER_TESTING_cmd_set_officer ("create-aml-officer-1",
+                                   NULL,
+                                   "Peter Falk",
+                                   true,
+                                   false),
+    TALER_TESTING_cmd_check_aml_decisions ("check-decisions-none-normal",
+                                           "create-aml-officer-1",
+                                           TALER_AML_NORMAL,
+                                           MHD_HTTP_NO_CONTENT),
+    TALER_TESTING_cmd_check_aml_decisions ("check-decisions-none-pending",
+                                           "create-aml-officer-1",
+                                           TALER_AML_PENDING,
+                                           MHD_HTTP_NO_CONTENT),
+    TALER_TESTING_cmd_check_aml_decisions ("check-decisions-none-frozen",
+                                           "create-aml-officer-1",
+                                           TALER_AML_FROZEN,
+                                           MHD_HTTP_NO_CONTENT),
+    TALER_TESTING_cmd_sleep ("sleep-1a",
+                             1),
+    TALER_TESTING_cmd_set_officer ("create-aml-officer-1-disable",
+                                   "create-aml-officer-1",
+                                   "Peter Falk",
+                                   true,
+                                   true),
+    /* Test that we are not allowed to take AML decisions as our
+       AML staff account is on read-only */
+    TALER_TESTING_cmd_take_aml_decision ("aml-decide-while-disabled",
+                                         "create-aml-officer-1",
+                                         "wallet-trigger-kyc-for-aml",
+                                         "EUR:10000",
+                                         "party time",
+                                         TALER_AML_NORMAL,
+                                         NULL,
+                                         MHD_HTTP_FORBIDDEN),
+    /* Check that no decision was taken, but that we are allowed
+       to read this information */
+    TALER_TESTING_cmd_check_aml_decision ("check-aml-decision-empty",
+                                          "create-aml-officer-1",
+                                          "aml-decide-while-disabled",
+                                          MHD_HTTP_NO_CONTENT),
+    TALER_TESTING_cmd_sleep ("sleep-1b",
+                             1),
+    TALER_TESTING_cmd_set_officer ("create-aml-officer-1-enable",
+                                   "create-aml-officer-1",
+                                   "Peter Falk",
+                                   true,
+                                   false),
+    TALER_TESTING_cmd_take_aml_decision ("aml-decide",
+                                         "create-aml-officer-1",
+                                         "wallet-trigger-kyc-for-aml",
+                                         "EUR:10000",
+                                         "party time",
+                                         TALER_AML_NORMAL,
+                                         NULL,
+                                         MHD_HTTP_NO_CONTENT),
+    TALER_TESTING_cmd_check_aml_decisions ("check-decisions-one-normal",
+                                           "create-aml-officer-1",
+                                           TALER_AML_NORMAL,
+                                           MHD_HTTP_OK),
+    TALER_TESTING_cmd_check_aml_decisions ("check-decisions-zero-frozen",
+                                           "create-aml-officer-1",
+                                           TALER_AML_FROZEN,
+                                           MHD_HTTP_NO_CONTENT),
+    TALER_TESTING_cmd_check_aml_decision ("check-aml-decision",
+                                          "create-aml-officer-1",
+                                          "aml-decide",
+                                          MHD_HTTP_OK),
+    TALER_TESTING_cmd_sleep ("sleep-1c",
+                             1),
+    TALER_TESTING_cmd_take_aml_decision ("aml-decide-freeze",
+                                         "create-aml-officer-1",
+                                         "wallet-trigger-kyc-for-aml",
+                                         "EUR:1000",
+                                         "party over",
+                                         TALER_AML_FROZEN,
+                                         NULL,
+                                         MHD_HTTP_NO_CONTENT),
+    TALER_TESTING_cmd_check_aml_decisions ("check-decisions-one-frozen",
+                                           "create-aml-officer-1",
+                                           TALER_AML_FROZEN,
+                                           MHD_HTTP_OK),
+    TALER_TESTING_cmd_check_aml_decisions ("check-decisions-zero-normal",
+                                           "create-aml-officer-1",
+                                           TALER_AML_NORMAL,
+                                           MHD_HTTP_NO_CONTENT),
+    TALER_TESTING_cmd_sleep ("sleep-1d",
+                             1),
+    TALER_TESTING_cmd_set_officer ("create-aml-officer-1-disable",
+                                   "create-aml-officer-1",
+                                   "Peter Falk",
+                                   false,
+                                   true),
+    /* Test that we are NOT allowed to read AML decisions now that
+       our AML staff account is disabled */
+    TALER_TESTING_cmd_check_aml_decision ("check-aml-decision-disabled",
+                                          "create-aml-officer-1",
+                                          "aml-decide",
+                                          MHD_HTTP_FORBIDDEN),
+    TALER_TESTING_cmd_end ()
+  };
 
   struct TALER_TESTING_Command commands[] = {
     TALER_TESTING_cmd_exec_offline_sign_fees ("offline-sign-fees",
@@ -436,6 +557,8 @@ run (void *cls,
                              push),
     TALER_TESTING_cmd_batch ("pull",
                              pull),
+    TALER_TESTING_cmd_batch ("aml",
+                             aml),
     TALER_TESTING_cmd_end ()
   };
 
