@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2018-2020 Taler Systems SA
+  Copyright (C) 2018-2021 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
@@ -82,7 +82,12 @@ struct DepositState
   /**
    * Refund deadline. Zero for no refunds.
    */
-  struct GNUNET_TIME_Absolute refund_deadline;
+  struct GNUNET_TIME_Timestamp refund_deadline;
+
+  /**
+   * Wire deadline.
+   */
+  struct GNUNET_TIME_Timestamp wire_deadline;
 
   /**
    * Set (by the interpreter) to a fresh private key.  This
@@ -98,7 +103,7 @@ struct DepositState
   /**
    * Timestamp of the /deposit operation in the wallet (contract signing time).
    */
-  struct GNUNET_TIME_Absolute wallet_timestamp;
+  struct GNUNET_TIME_Timestamp wallet_timestamp;
 
   /**
    * Interpreter state.
@@ -134,7 +139,7 @@ struct DepositState
   /**
    * When did the exchange receive the deposit?
    */
-  struct GNUNET_TIME_Absolute exchange_timestamp;
+  struct GNUNET_TIME_Timestamp exchange_timestamp;
 
   /**
    * Signing key used by the exchange to sign the
@@ -207,43 +212,36 @@ do_retry (void *cls)
  * check if the response code is acceptable.
  *
  * @param cls closure.
- * @param hr HTTP response details
- * @param exchange_timestamp when did the exchange receive the deposit permission
- * @param exchange_sig signature provided by the exchange
- *        (NULL on errors)
- * @param exchange_pub public key of the exchange,
- *        used for signing the response.
+ * @param dr deposit response details
  */
 static void
 deposit_cb (void *cls,
-            const struct TALER_EXCHANGE_HttpResponse *hr,
-            const struct GNUNET_TIME_Absolute exchange_timestamp,
-            const struct TALER_ExchangeSignatureP *exchange_sig,
-            const struct TALER_ExchangePublicKeyP *exchange_pub)
+            const struct TALER_EXCHANGE_DepositResult *dr)
 {
   struct DepositState *ds = cls;
 
   ds->dh = NULL;
-  if (ds->expected_response_code != hr->http_status)
+  if (ds->expected_response_code != dr->hr.http_status)
   {
     if (0 != ds->do_retry)
     {
       ds->do_retry--;
-      if ( (0 == hr->http_status) ||
-           (TALER_EC_GENERIC_DB_SOFT_FAILURE == hr->ec) ||
-           (MHD_HTTP_INTERNAL_SERVER_ERROR == hr->http_status) )
+      if ( (0 == dr->hr.http_status) ||
+           (TALER_EC_GENERIC_DB_SOFT_FAILURE == dr->hr.ec) ||
+           (MHD_HTTP_INTERNAL_SERVER_ERROR == dr->hr.http_status) )
       {
         GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                     "Retrying deposit failed with %u/%d\n",
-                    hr->http_status,
-                    (int) hr->ec);
+                    dr->hr.http_status,
+                    (int) dr->hr.ec);
         /* on DB conflicts, do not use backoff */
-        if (TALER_EC_GENERIC_DB_SOFT_FAILURE == hr->ec)
+        if (TALER_EC_GENERIC_DB_SOFT_FAILURE == dr->hr.ec)
           ds->backoff = GNUNET_TIME_UNIT_ZERO;
         else
           ds->backoff = GNUNET_TIME_randomized_backoff (ds->backoff,
                                                         MAX_BACKOFF);
         ds->is->commands[ds->is->ip].num_tries++;
+        GNUNET_assert (NULL == ds->retry_task);
         ds->retry_task
           = GNUNET_SCHEDULER_add_delayed (ds->backoff,
                                           &do_retry,
@@ -253,22 +251,22 @@ deposit_cb (void *cls,
     }
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Unexpected response code %u to command %s in %s:%u\n",
-                hr->http_status,
+                dr->hr.http_status,
                 ds->is->commands[ds->is->ip].label,
                 __FILE__,
                 __LINE__);
-    json_dumpf (hr->reply,
+    json_dumpf (dr->hr.reply,
                 stderr,
-                0);
+                JSON_INDENT (2));
     TALER_TESTING_interpreter_fail (ds->is);
     return;
   }
-  if (MHD_HTTP_OK == hr->http_status)
+  if (MHD_HTTP_OK == dr->hr.http_status)
   {
     ds->deposit_succeeded = GNUNET_YES;
-    ds->exchange_timestamp = exchange_timestamp;
-    ds->exchange_pub = *exchange_pub;
-    ds->exchange_sig = *exchange_sig;
+    ds->exchange_timestamp = dr->details.success.deposit_timestamp;
+    ds->exchange_pub = *dr->details.success.exchange_pub;
+    ds->exchange_sig = *dr->details.success.exchange_sig;
   }
   TALER_TESTING_interpreter_next (ds->is);
 }
@@ -290,12 +288,23 @@ deposit_run (void *cls,
   const struct TALER_TESTING_Command *coin_cmd;
   const struct TALER_CoinSpendPrivateKeyP *coin_priv;
   struct TALER_CoinSpendPublicKeyP coin_pub;
+  const struct TALER_AgeCommitmentProof *age_commitment_proof = NULL;
+  struct TALER_AgeCommitmentHash h_age_commitment = {0};
   const struct TALER_EXCHANGE_DenomPublicKey *denom_pub;
   const struct TALER_DenominationSignature *denom_pub_sig;
   struct TALER_CoinSpendSignatureP coin_sig;
-  struct GNUNET_TIME_Absolute wire_deadline;
   struct TALER_MerchantPublicKeyP merchant_pub;
-  struct GNUNET_HashCode h_contract_terms;
+  struct TALER_PrivateContractHashP h_contract_terms;
+  enum TALER_ErrorCode ec;
+  struct TALER_WireSaltP wire_salt;
+  const char *payto_uri;
+  struct GNUNET_JSON_Specification spec[] = {
+    GNUNET_JSON_spec_string ("payto_uri",
+                             &payto_uri),
+    GNUNET_JSON_spec_fixed_auto ("salt",
+                                 &wire_salt),
+    GNUNET_JSON_spec_end ()
+  };
 
   (void) cmd;
   ds->is = is;
@@ -317,6 +326,7 @@ deposit_run (void *cls,
     ds->coin_reference = ods->coin_reference;
     ds->coin_index = ods->coin_index;
     ds->wire_details = json_incref (ods->wire_details);
+    GNUNET_assert (NULL != ds->wire_details);
     ds->contract_terms = json_incref (ods->contract_terms);
     ds->wallet_timestamp = ods->wallet_timestamp;
     ds->refund_deadline = ods->refund_deadline;
@@ -340,7 +350,6 @@ deposit_run (void *cls,
     }
     if ( (GNUNET_OK !=
           TALER_TESTING_get_trait_merchant_priv (cmd,
-                                                 0,
                                                  &merchant_priv)) )
     {
       GNUNET_break (0);
@@ -349,6 +358,19 @@ deposit_run (void *cls,
     }
     ds->merchant_priv = *merchant_priv;
   }
+  GNUNET_assert (NULL != ds->wire_details);
+  if (GNUNET_OK !=
+      GNUNET_JSON_parse (ds->wire_details,
+                         spec,
+                         NULL, NULL))
+  {
+    json_dumpf (ds->wire_details,
+                stderr,
+                JSON_INDENT (2));
+    GNUNET_break (0);
+    TALER_TESTING_interpreter_fail (is);
+    return;
+  }
   GNUNET_assert (ds->coin_reference);
   coin_cmd = TALER_TESTING_interpreter_lookup_command (is,
                                                        ds->coin_reference);
@@ -363,6 +385,10 @@ deposit_run (void *cls,
         TALER_TESTING_get_trait_coin_priv (coin_cmd,
                                            ds->coin_index,
                                            &coin_priv)) ||
+       (GNUNET_OK !=
+        TALER_TESTING_get_trait_age_commitment_proof (coin_cmd,
+                                                      ds->coin_index,
+                                                      &age_commitment_proof)) ||
        (GNUNET_OK !=
         TALER_TESTING_get_trait_denom_pub (coin_cmd,
                                            ds->coin_index,
@@ -379,60 +405,88 @@ deposit_run (void *cls,
     TALER_TESTING_interpreter_fail (is);
     return;
   }
-  ds->deposit_fee = denom_pub->fee_deposit;
+
+  if (NULL != age_commitment_proof)
+  {
+    TALER_age_commitment_hash (&age_commitment_proof->commitment,
+                               &h_age_commitment);
+  }
+  ds->deposit_fee = denom_pub->fees.deposit;
   GNUNET_CRYPTO_eddsa_key_get_public (&coin_priv->eddsa_priv,
                                       &coin_pub.eddsa_pub);
 
-  if (0 != ds->refund_deadline.abs_value_us)
+  if (! GNUNET_TIME_absolute_is_zero (ds->refund_deadline.abs_time))
   {
     struct GNUNET_TIME_Relative refund_deadline;
 
-    refund_deadline = GNUNET_TIME_absolute_get_remaining (ds->refund_deadline);
-    wire_deadline = GNUNET_TIME_relative_to_absolute
-                      (GNUNET_TIME_relative_multiply (refund_deadline, 2));
+    refund_deadline
+      = GNUNET_TIME_absolute_get_remaining (ds->refund_deadline.abs_time);
+    ds->wire_deadline
+      =
+        GNUNET_TIME_relative_to_timestamp (
+          GNUNET_TIME_relative_multiply (refund_deadline,
+                                         2));
   }
   else
   {
     ds->refund_deadline = ds->wallet_timestamp;
-    wire_deadline = GNUNET_TIME_relative_to_absolute (GNUNET_TIME_UNIT_ZERO);
+    ds->wire_deadline = GNUNET_TIME_timestamp_get ();
   }
   GNUNET_CRYPTO_eddsa_key_get_public (&ds->merchant_priv.eddsa_priv,
                                       &merchant_pub.eddsa_pub);
-  (void) GNUNET_TIME_round_abs (&wire_deadline);
   {
-    struct GNUNET_HashCode h_wire;
+    struct TALER_MerchantWireHashP h_wire;
 
     GNUNET_assert (GNUNET_OK ==
                    TALER_JSON_merchant_wire_signature_hash (ds->wire_details,
                                                             &h_wire));
-    TALER_EXCHANGE_deposit_permission_sign (&ds->amount,
-                                            &denom_pub->fee_deposit,
-                                            &h_wire,
-                                            &h_contract_terms,
-                                            &denom_pub->h_key,
-                                            coin_priv,
-                                            ds->wallet_timestamp,
-                                            &merchant_pub,
-                                            ds->refund_deadline,
-                                            &coin_sig);
+    TALER_wallet_deposit_sign (&ds->amount,
+                               &denom_pub->fees.deposit,
+                               &h_wire,
+                               &h_contract_terms,
+                               &h_age_commitment,
+                               NULL, /* FIXME #7270: add hash of extensions */
+                               &denom_pub->h_key,
+                               ds->wallet_timestamp,
+                               &merchant_pub,
+                               ds->refund_deadline,
+                               coin_priv,
+                               &coin_sig);
+  }
+  GNUNET_assert (NULL == ds->dh);
+  {
+    struct TALER_EXCHANGE_CoinDepositDetail cdd = {
+      .amount = ds->amount,
+      .h_age_commitment = h_age_commitment,
+      .coin_pub = coin_pub,
+      .coin_sig = coin_sig,
+      .denom_sig = *denom_pub_sig,
+      .h_denom_pub = denom_pub->h_key
+    };
+    struct TALER_EXCHANGE_DepositContractDetail dcd = {
+      .wire_deadline = ds->wire_deadline,
+      .merchant_payto_uri = payto_uri,
+      .wire_salt = wire_salt,
+      .h_contract_terms = h_contract_terms,
+      .policy_details = NULL /* FIXME #7270-OEC */,
+      .timestamp = ds->wallet_timestamp,
+      .merchant_pub = merchant_pub,
+      .refund_deadline = ds->refund_deadline
+    };
+
+    ds->dh = TALER_EXCHANGE_deposit (is->exchange,
+                                     &dcd,
+                                     &cdd,
+                                     &deposit_cb,
+                                     ds,
+                                     &ec);
   }
-  ds->dh = TALER_EXCHANGE_deposit (is->exchange,
-                                   &ds->amount,
-                                   wire_deadline,
-                                   ds->wire_details,
-                                   &h_contract_terms,
-                                   &coin_pub,
-                                   denom_pub_sig,
-                                   &denom_pub->key,
-                                   ds->wallet_timestamp,
-                                   &merchant_pub,
-                                   ds->refund_deadline,
-                                   &coin_sig,
-                                   &deposit_cb,
-                                   ds);
   if (NULL == ds->dh)
   {
     GNUNET_break (0);
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Could not create deposit with EC %d\n",
+                (int) ec);
     TALER_TESTING_interpreter_fail (is);
     return;
   }
@@ -479,10 +533,9 @@ deposit_cleanup (void *cls,
  * @param[out] ret result.
  * @param trait name of the trait.
  * @param index index number of the object to offer.
- *
  * @return #GNUNET_OK on success.
  */
-static int
+static enum GNUNET_GenericReturnValue
 deposit_traits (void *cls,
                 const void **ret,
                 const char *trait,
@@ -492,6 +545,7 @@ deposit_traits (void *cls,
   const struct TALER_TESTING_Command *coin_cmd;
   /* Will point to coin cmd internals. */
   const struct TALER_CoinSpendPrivateKeyP *coin_spent_priv;
+  const struct TALER_AgeCommitmentProof *age_commitment_proof;
 
   if (GNUNET_YES != ds->command_initialized)
   {
@@ -509,10 +563,14 @@ deposit_traits (void *cls,
     TALER_TESTING_interpreter_fail (ds->is);
     return GNUNET_NO;
   }
-  if (GNUNET_OK !=
-      TALER_TESTING_get_trait_coin_priv (coin_cmd,
-                                         ds->coin_index,
-                                         &coin_spent_priv))
+  if ( (GNUNET_OK !=
+        TALER_TESTING_get_trait_coin_priv (coin_cmd,
+                                           ds->coin_index,
+                                           &coin_spent_priv)) ||
+       (GNUNET_OK !=
+        TALER_TESTING_get_trait_age_commitment_proof (coin_cmd,
+                                                      ds->coin_index,
+                                                      &age_commitment_proof)) )
   {
     GNUNET_break (0);
     TALER_TESTING_interpreter_fail (ds->is);
@@ -529,20 +587,21 @@ deposit_traits (void *cls,
       /* These traits are always available */
       TALER_TESTING_make_trait_coin_priv (0,
                                           coin_spent_priv),
-      TALER_TESTING_make_trait_wire_details (0,
-                                             ds->wire_details),
-      TALER_TESTING_make_trait_contract_terms (0,
-                                               ds->contract_terms),
-      TALER_TESTING_make_trait_merchant_priv (0,
-                                              &ds->merchant_priv),
-      TALER_TESTING_make_trait_amount_obj (
-        TALER_TESTING_CMD_DEPOSIT_TRAIT_IDX_DEPOSIT_VALUE,
-        &ds->amount),
-      TALER_TESTING_make_trait_amount_obj (
-        TALER_TESTING_CMD_DEPOSIT_TRAIT_IDX_DEPOSIT_FEE,
-        &ds->deposit_fee),
-      TALER_TESTING_make_trait_absolute_time (0,
-                                              &ds->exchange_timestamp),
+      TALER_TESTING_make_trait_age_commitment_proof (0,
+                                                     age_commitment_proof),
+      TALER_TESTING_make_trait_wire_details (ds->wire_details),
+      TALER_TESTING_make_trait_contract_terms (ds->contract_terms),
+      TALER_TESTING_make_trait_merchant_priv (&ds->merchant_priv),
+      TALER_TESTING_make_trait_deposit_amount (0,
+                                               &ds->amount),
+      TALER_TESTING_make_trait_deposit_fee_amount (0,
+                                                   &ds->deposit_fee),
+      TALER_TESTING_make_trait_timestamp (0,
+                                          &ds->exchange_timestamp),
+      TALER_TESTING_make_trait_wire_deadline (0,
+                                              &ds->wire_deadline),
+      TALER_TESTING_make_trait_refund_deadline (0,
+                                                &ds->refund_deadline),
       TALER_TESTING_trait_end ()
     };
 
@@ -556,28 +615,6 @@ deposit_traits (void *cls,
 }
 
 
-/**
- * Create a "deposit" command.
- *
- * @param label command label.
- * @param coin_reference reference to any operation that can
- *        provide a coin.
- * @param coin_index if @a withdraw_reference offers an array of
- *        coins, this parameter selects which one in that array.
- *        This value is currently ignored, as only one-coin
- *        withdrawals are implemented.
- * @param target_account_payto target account for the "deposit"
- *        request.
- * @param contract_terms contract terms to be signed over by the
- *        coin.
- * @param refund_deadline refund deadline, zero means 'no refunds'.
- *        Note, if time were absolute, then it would have come
- *        one day and disrupt tests meaning.
- * @param amount how much is going to be deposited.
- * @param expected_response_code expected HTTP response code.
- *
- * @return the command.
- */
 struct TALER_TESTING_Command
 TALER_TESTING_cmd_deposit (const char *label,
                            const char *coin_reference,
@@ -589,13 +626,12 @@ TALER_TESTING_cmd_deposit (const char *label,
                            unsigned int expected_response_code)
 {
   struct DepositState *ds;
-  json_t *wire_details;
 
-  wire_details = TALER_TESTING_make_wire_details (target_account_payto);
   ds = GNUNET_new (struct DepositState);
   ds->coin_reference = coin_reference;
   ds->coin_index = coin_index;
-  ds->wire_details = wire_details;
+  ds->wire_details = TALER_TESTING_make_wire_details (target_account_payto);
+  GNUNET_assert (NULL != ds->wire_details);
   ds->contract_terms = json_loads (contract_terms,
                                    JSON_REJECT_DUPLICATES,
                                    NULL);
@@ -608,19 +644,20 @@ TALER_TESTING_cmd_deposit (const char *label,
                 label);
     GNUNET_assert (0);
   }
-  ds->wallet_timestamp = GNUNET_TIME_absolute_get ();
-  (void) GNUNET_TIME_round_abs (&ds->wallet_timestamp);
-
-  json_object_set_new (ds->contract_terms,
-                       "timestamp",
-                       GNUNET_JSON_from_time_abs (ds->wallet_timestamp));
-  if (0 != refund_deadline.rel_value_us)
+  ds->wallet_timestamp = GNUNET_TIME_timestamp_get ();
+  GNUNET_assert (0 ==
+                 json_object_set_new (ds->contract_terms,
+                                      "timestamp",
+                                      GNUNET_JSON_from_timestamp (
+                                        ds->wallet_timestamp)));
+  if (! GNUNET_TIME_relative_is_zero (refund_deadline))
   {
-    ds->refund_deadline = GNUNET_TIME_relative_to_absolute (refund_deadline);
-    (void) GNUNET_TIME_round_abs (&ds->refund_deadline);
-    json_object_set_new (ds->contract_terms,
-                         "refund_deadline",
-                         GNUNET_JSON_from_time_abs (ds->refund_deadline));
+    ds->refund_deadline = GNUNET_TIME_relative_to_timestamp (refund_deadline);
+    GNUNET_assert (0 ==
+                   json_object_set_new (ds->contract_terms,
+                                        "refund_deadline",
+                                        GNUNET_JSON_from_timestamp (
+                                          ds->refund_deadline)));
   }
   GNUNET_assert (GNUNET_OK ==
                  TALER_string_to_amount (amount,
@@ -641,30 +678,6 @@ TALER_TESTING_cmd_deposit (const char *label,
 }
 
 
-/**
- * Create a "deposit" command that references an existing merchant key.
- *
- * @param label command label.
- * @param coin_reference reference to any operation that can
- *        provide a coin.
- * @param coin_index if @a withdraw_reference offers an array of
- *        coins, this parameter selects which one in that array.
- *        This value is currently ignored, as only one-coin
- *        withdrawals are implemented.
- * @param target_account_payto target account for the "deposit"
- *        request.
- * @param contract_terms contract terms to be signed over by the
- *        coin.
- * @param refund_deadline refund deadline, zero means 'no refunds'.
- *        Note, if time were absolute, then it would have come
- *        one day and disrupt tests meaning.
- * @param amount how much is going to be deposited.
- * @param expected_response_code expected HTTP response code.
- * @param merchant_priv_reference reference to another operation
- *        that has a merchant private key trait
- *
- * @return the command.
- */
 struct TALER_TESTING_Command
 TALER_TESTING_cmd_deposit_with_ref (const char *label,
                                     const char *coin_reference,
@@ -677,14 +690,13 @@ TALER_TESTING_cmd_deposit_with_ref (const char *label,
                                     const char *merchant_priv_reference)
 {
   struct DepositState *ds;
-  json_t *wire_details;
 
-  wire_details = TALER_TESTING_make_wire_details (target_account_payto);
   ds = GNUNET_new (struct DepositState);
   ds->merchant_priv_reference = merchant_priv_reference;
   ds->coin_reference = coin_reference;
   ds->coin_index = coin_index;
-  ds->wire_details = wire_details;
+  ds->wire_details = TALER_TESTING_make_wire_details (target_account_payto);
+  GNUNET_assert (NULL != ds->wire_details);
   ds->contract_terms = json_loads (contract_terms,
                                    JSON_REJECT_DUPLICATES,
                                    NULL);
@@ -696,19 +708,20 @@ TALER_TESTING_cmd_deposit_with_ref (const char *label,
                 label);
     GNUNET_assert (0);
   }
-  ds->wallet_timestamp = GNUNET_TIME_absolute_get ();
-  (void) GNUNET_TIME_round_abs (&ds->wallet_timestamp);
-
-  json_object_set_new (ds->contract_terms,
-                       "timestamp",
-                       GNUNET_JSON_from_time_abs (ds->wallet_timestamp));
+  ds->wallet_timestamp = GNUNET_TIME_timestamp_get ();
+  GNUNET_assert (0 ==
+                 json_object_set_new (ds->contract_terms,
+                                      "timestamp",
+                                      GNUNET_JSON_from_timestamp (
+                                        ds->wallet_timestamp)));
   if (0 != refund_deadline.rel_value_us)
   {
-    ds->refund_deadline = GNUNET_TIME_relative_to_absolute (refund_deadline);
-    (void) GNUNET_TIME_round_abs (&ds->refund_deadline);
-    json_object_set_new (ds->contract_terms,
-                         "refund_deadline",
-                         GNUNET_JSON_from_time_abs (ds->refund_deadline));
+    ds->refund_deadline = GNUNET_TIME_relative_to_timestamp (refund_deadline);
+    GNUNET_assert (0 ==
+                   json_object_set_new (ds->contract_terms,
+                                        "refund_deadline",
+                                        GNUNET_JSON_from_timestamp (
+                                          ds->refund_deadline)));
   }
   GNUNET_assert (GNUNET_OK ==
                  TALER_string_to_amount (amount,
@@ -729,15 +742,6 @@ TALER_TESTING_cmd_deposit_with_ref (const char *label,
 }
 
 
-/**
- * Create a "deposit" command that repeats an existing
- * deposit command.
- *
- * @param label command label.
- * @param deposit_reference which deposit command should we repeat
- * @param expected_response_code expected HTTP response code.
- * @return the command.
- */
 struct TALER_TESTING_Command
 TALER_TESTING_cmd_deposit_replay (const char *label,
                                   const char *deposit_reference,
@@ -762,13 +766,6 @@ TALER_TESTING_cmd_deposit_replay (const char *label,
 }
 
 
-/**
- * Modify a deposit command to enable retries when we get transient
- * errors from the exchange.
- *
- * @param cmd a deposit command
- * @return the command with retries enabled
- */
 struct TALER_TESTING_Command
 TALER_TESTING_cmd_deposit_with_retry (struct TALER_TESTING_Command cmd)
 {
