@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2018-2021 Taler Systems SA
+  Copyright (C) 2018-2023 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as
@@ -92,11 +92,10 @@ TALER_TESTING_run_libeufin (const struct TALER_TESTING_BankConfiguration *bc)
   struct TALER_TESTING_LibeufinServices ret = { 0 };
   unsigned int iter;
   char *curl_check_cmd;
-  const char *db_conn = "jdbc:sqlite:/tmp/libeufin-exchange-test.sqlite3";
 
   setenv (
     "LIBEUFIN_NEXUS_DB_CONNECTION",
-    db_conn,
+    "jdbc:sqlite:/tmp/libeufin-exchange-test-nexusdb.sqlite3",
     1); // not overwriting any potentially existing DB.
 
   nexus_proc = GNUNET_OS_start_process (
@@ -146,8 +145,8 @@ TALER_TESTING_run_libeufin (const struct TALER_TESTING_BankConfiguration *bc)
   fprintf (stderr, "\n");
   setenv (
     "LIBEUFIN_SANDBOX_DB_CONNECTION",
-    db_conn,
-    1); // not overwriting existing any potentially existing DB.
+    "jdbc:sqlite:/tmp/libeufin-exchange-test-sandboxdb.sqlite3",
+    1); // not overwriting any potentially existing DB.
   setenv (
     "LIBEUFIN_SANDBOX_ADMIN_PASSWORD",
     "secret",
@@ -299,10 +298,10 @@ TALER_TESTING_run_bank (const char *config_filename,
 
 
 enum GNUNET_GenericReturnValue
-TALER_TESTING_prepare_nexus (const char *config_filename,
-                             int reset_db,
-                             const char *config_section,
-                             struct TALER_TESTING_BankConfiguration *bc)
+TALER_TESTING_prepare_libeufin (const char *config_filename,
+                                bool reset_db,
+                                const char *config_section,
+                                struct TALER_TESTING_BankConfiguration *bc)
 {
   struct GNUNET_CONFIGURATION_Handle *cfg;
   unsigned long long port;
@@ -355,9 +354,9 @@ TALER_TESTING_prepare_nexus (const char *config_filename,
       GNUNET_NETWORK_test_port_free (IPPROTO_TCP,
                                      (uint16_t) port))
   {
-    fprintf (stderr,
-             "Required port %llu not available, skipping.\n",
-             port);
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Required port %llu not available, skipping.\n",
+                port);
     GNUNET_break (0);
     GNUNET_free (database);
     GNUNET_CONFIGURATION_destroy (cfg);
@@ -365,12 +364,20 @@ TALER_TESTING_prepare_nexus (const char *config_filename,
   }
 
   /* DB preparation */
-  if (GNUNET_YES == reset_db)
+  if (reset_db)
   {
-    if (0 != system ("rm -f /tmp/libeufin-exchange-test.sqlite3"))
+    if (0 != system ("rm -f /tmp/libeufin-exchange-test-nexusdb.sqlite3"))
     {
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                  "Failed to invoke db-removal command.\n");
+                  "Failed to invoke db-removal command on nexusdb.\n");
+      GNUNET_free (database);
+      GNUNET_CONFIGURATION_destroy (cfg);
+      return GNUNET_SYSERR;
+    }
+    if (0 != system ("rm -f /tmp/libeufin-exchange-test-sandboxdb.sqlite3"))
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                  "Failed to invoke db-removal command on sandboxdb.\n");
       GNUNET_free (database);
       GNUNET_CONFIGURATION_destroy (cfg);
       return GNUNET_SYSERR;
@@ -407,11 +414,14 @@ TALER_TESTING_prepare_nexus (const char *config_filename,
               "Relying on nexus %s on port %u\n",
               bc->exchange_auth.wire_gateway_url,
               (unsigned int) port);
-  GNUNET_log (GNUNET_ERROR_TYPE_INFO, "exchange payto: %s\n",
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "exchange payto: %s\n",
               bc->exchange_payto);
-  GNUNET_log (GNUNET_ERROR_TYPE_INFO, "user42_payto: %s\n",
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "user42_payto: %s\n",
               bc->user42_payto);
-  GNUNET_log (GNUNET_ERROR_TYPE_INFO, "user42_payto: %s\n",
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "user42_payto: %s\n",
               bc->user43_payto);
   return GNUNET_OK;
 }
@@ -419,7 +429,7 @@ TALER_TESTING_prepare_nexus (const char *config_filename,
 
 enum GNUNET_GenericReturnValue
 TALER_TESTING_prepare_bank (const char *config_filename,
-                            int reset_db,
+                            bool reset_db,
                             const char *config_section,
                             struct TALER_TESTING_BankConfiguration *bc)
 {
@@ -491,9 +501,9 @@ TALER_TESTING_prepare_bank (const char *config_filename,
       GNUNET_NETWORK_test_port_free (IPPROTO_TCP,
                                      (uint16_t) port))
   {
-    fprintf (stderr,
-             "Required port %llu not available, skipping.\n",
-             port);
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Required port %llu not available, skipping.\n",
+                port);
     GNUNET_break (0);
     GNUNET_free (database);
     GNUNET_CONFIGURATION_destroy (cfg);
@@ -501,7 +511,7 @@ TALER_TESTING_prepare_bank (const char *config_filename,
   }
 
   /* DB preparation */
-  if (GNUNET_YES == reset_db)
+  if (reset_db)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                 "Flushing bank database\n");
@@ -582,7 +592,7 @@ TALER_TESTING_prepare_bank (const char *config_filename,
   bc->user42_payto = "payto://x-taler-bank/localhost/42?receiver-name=42";
   bc->user43_payto = "payto://x-taler-bank/localhost/43?receiver-name=43";
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-              "Using pybank %s on port %u\n",
+              "Using bank %s on port %u\n",
               bc->exchange_auth.wire_gateway_url,
               (unsigned int) port);
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
