@@ -26,8 +26,7 @@
 GNUNET_NETWORK_STRUCT_BEGIN
 
 /**
- * @brief Format used to generate the signature on a request to deposit
- * a coin into the account of a merchant.
+ * @brief Format used to generate the signature on an AML decision.
  */
 struct TALER_AmlDecisionPS
 {
@@ -57,6 +56,12 @@ struct TALER_AmlDecisionPS
    */
   struct TALER_PaytoHashP h_payto GNUNET_PACKED;
 
+  /**
+   * Hash over JSON array with KYC requirements that were imposed. All zeros
+   * for none.
+   */
+  struct GNUNET_HashCode h_kyc_requirements;
+
   /**
    * What is the new AML status?
    */
@@ -73,6 +78,7 @@ TALER_officer_aml_decision_sign (
   const struct TALER_Amount *new_threshold,
   const struct TALER_PaytoHashP *h_payto,
   enum TALER_AmlDecisionState new_state,
+  const json_t *kyc_requirements,
   const struct TALER_AmlOfficerPrivateKeyP *officer_priv,
   struct TALER_AmlOfficerSignatureP *officer_sig)
 {
@@ -88,6 +94,9 @@ TALER_officer_aml_decision_sign (
                       &ad.h_justification);
   TALER_amount_hton (&ad.new_threshold,
                      new_threshold);
+  if (NULL != kyc_requirements)
+    TALER_json_hash (kyc_requirements,
+                     &ad.h_kyc_requirements);
   GNUNET_CRYPTO_eddsa_sign (&officer_priv->eddsa_priv,
                             &ad,
                             &officer_sig->eddsa_signature);
@@ -101,6 +110,7 @@ TALER_officer_aml_decision_verify (
   const struct TALER_Amount *new_threshold,
   const struct TALER_PaytoHashP *h_payto,
   enum TALER_AmlDecisionState new_state,
+  const json_t *kyc_requirements,
   const struct TALER_AmlOfficerPublicKeyP *officer_pub,
   const struct TALER_AmlOfficerSignatureP *officer_sig)
 {
@@ -116,6 +126,9 @@ TALER_officer_aml_decision_verify (
                       &ad.h_justification);
   TALER_amount_hton (&ad.new_threshold,
                      new_threshold);
+  if (NULL != kyc_requirements)
+    TALER_json_hash (kyc_requirements,
+                     &ad.h_kyc_requirements);
   return GNUNET_CRYPTO_eddsa_verify (
     TALER_SIGNATURE_AML_DECISION,
     &ad,
@@ -124,4 +137,63 @@ TALER_officer_aml_decision_verify (
 }
 
 
+GNUNET_NETWORK_STRUCT_BEGIN
+
+/**
+ * @brief Format used to generate the signature on any AML query.
+ */
+struct TALER_AmlQueryPS
+{
+  /**
+   * Purpose must be #TALER_SIGNATURE_AML_QUERY.
+   * Used for an EdDSA signature with the `struct TALER_AmlOfficerPublicKeyP`.
+   */
+  struct GNUNET_CRYPTO_EccSignaturePurpose purpose;
+
+};
+
+GNUNET_NETWORK_STRUCT_END
+
+
+void
+TALER_officer_aml_query_sign (
+  const struct TALER_AmlOfficerPrivateKeyP *officer_priv,
+  struct TALER_AmlOfficerSignatureP *officer_sig)
+{
+  struct TALER_AmlQueryPS aq = {
+    .purpose.purpose = htonl (TALER_SIGNATURE_AML_QUERY),
+    .purpose.size = htonl (sizeof (aq))
+  };
+
+  GNUNET_CRYPTO_eddsa_sign (&officer_priv->eddsa_priv,
+                            &aq,
+                            &officer_sig->eddsa_signature);
+}
+
+
+/**
+ * Verify AML query authorization.
+ *
+ * @param officer_pub public key of AML officer
+ * @param officer_sig signature to verify
+ * @return #GNUNET_OK if the signature is valid
+ */
+enum GNUNET_GenericReturnValue
+TALER_officer_aml_query_verify (
+  const struct TALER_AmlOfficerPublicKeyP *officer_pub,
+  const struct TALER_AmlOfficerSignatureP *officer_sig)
+{
+  struct TALER_AmlQueryPS aq = {
+    .purpose.purpose = htonl (TALER_SIGNATURE_AML_QUERY),
+    .purpose.size = htonl (sizeof (aq))
+  };
+
+  return GNUNET_CRYPTO_eddsa_verify (
+    TALER_SIGNATURE_AML_QUERY,
+    &aq,
+    &officer_sig->eddsa_signature,
+    &officer_pub->eddsa_pub);
+}
+
+
 /* end of aml_signatures.c */
